# -*- coding: UTF-8 -*-

#########################################################
# Name: audiodialogs.py
# Porpose: Dialog for choice and settings audio parameters
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: Dec.28.2018, Sept.03.2019
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

import wx
import webbrowser

class AudioSettings(wx.Dialog):
    """
    Provides a dialog for audio settings which bit-rate, sample-rate,
    audio-channels and bit-per-sample (bit depth).
    """
    def __init__(self, parent, audio_type, arate, 
                 adepth, abitrate, achannel, title):
        """
        The given 'audio_type' parameter is a string that represents the 
        audio format without punctuation, which it is passed creating the 
        instance at 'datastr = TypeAudioParameters(audio_type)' class.
        This class has the same attributes as the TypeAudioParameters class 
        but here they are assigned by reference with the instance-object.
        """
        wx.Dialog.__init__(self, parent, -1, title=title, 
                           style=wx.DEFAULT_DIALOG_STYLE
                           )
        datastr = TypeAudioParameters(audio_type)# instance for audio param
        # set attributes:
        self.sample_rate = datastr.sample_rate
        self.channels = datastr.channels
        self.bitrate = datastr.bitrate
        self.bitdepth = datastr.bitdepth
        samplerate_list = []
        channel_list = []
        bitrate_list = []
        bitdepth_list = []

        if self.bitrate == None:
            self.bitrate = {0:('not applicable ',"")}
        
        if self.bitdepth == None:
            self.bitdepth = {0:('not applicable ',"")}

        for a in self.sample_rate.values():
            samplerate_list.append(a[0])
            
        for b in self.channels.values():
            channel_list.append(b[0])
            
        for c in self.bitrate.values():
            bitrate_list.append(c[0])
            
        for d in self.bitdepth.values():
            bitdepth_list.append(d[0])

        self.rdb_bitrate = wx.RadioBox(self, wx.ID_ANY, (
            "Audio Bit-Rate"), choices=bitrate_list, majorDimension=0, 
             style=wx.RA_SPECIFY_ROWS)

        self.rdb_channels = wx.RadioBox(self, wx.ID_ANY, (
            "Audio Channel"), choices=channel_list, majorDimension=0, 
             style=wx.RA_SPECIFY_ROWS)

        self.rdb_sample_r = wx.RadioBox(self, wx.ID_ANY, (
            "Audio Rate (sample rate)"), choices=samplerate_list, 
             majorDimension=0, style=wx.RA_SPECIFY_ROWS)
        
        self.rdb_bitdepth = wx.RadioBox(self, wx.ID_ANY, (
            "Bit per Sample (bit depth)"), choices=bitdepth_list, 
             majorDimension=0, style=wx.RA_SPECIFY_ROWS)
        
        if self.rdb_bitrate.GetStringSelection() == 'not applicable ':
            self.rdb_bitrate.Disable()
            
        if self.rdb_bitdepth.GetStringSelection() == 'not applicable ':
            self.rdb_bitdepth.Disable()
            
        btn_help = wx.Button(self, wx.ID_HELP, "", size=(-1, -1))
        self.btn_cancel = wx.Button(self, wx.ID_CANCEL, "")
        self.btn_ok = wx.Button(self, wx.ID_OK, "")
        btn_reset = wx.Button(self, wx.ID_CLEAR, "")
        
        """----------------------Properties----------------------"""
        self.rdb_bitrate.SetSelection(0)
        self.rdb_bitrate.SetToolTip(datastr.bitrate_tooltip)
        self.rdb_channels.SetSelection(0)
        self.rdb_channels.SetToolTip(datastr.channel_tooltip)
        self.rdb_sample_r.SetSelection(0)
        self.rdb_sample_r.SetToolTip(datastr.sample_rate_tooltip)
        self.rdb_bitdepth.SetSelection(0)
        self.rdb_bitdepth.SetToolTip(datastr.bitdepth_tooltip)
        # Set previusly settings:
        if arate[0]:
            self.rdb_sample_r.SetSelection(samplerate_list.index(arate[0]))
        if adepth[0]:
            self.rdb_bitdepth.SetSelection(bitdepth_list.index(adepth[0]))
        if abitrate[0]:
            self.rdb_bitrate.SetSelection(bitrate_list.index(abitrate[0]))
        if achannel[0]:
            self.rdb_channels.SetSelection(channel_list.index(achannel[0]))
            
        """----------------------Build layout----------------------"""
        sizerBase = wx.BoxSizer(wx.VERTICAL)
        grid_sizer_1 = wx.FlexGridSizer(1, 4, 0, 0)#radiobox
        sizerBase.Add(grid_sizer_1, 0, wx.ALL, 0)
        grid_sizer_1.Add(self.rdb_bitrate, 0, wx.ALL, 15)
        grid_sizer_1.Add(self.rdb_channels, 0, wx.ALL, 15)
        grid_sizer_1.Add(self.rdb_sample_r, 0, wx.ALL, 15)
        grid_sizer_1.Add(self.rdb_bitdepth, 0, wx.ALL, 15)
        
        gridhelp = wx.GridSizer(1, 1, 0, 0)#buttons
        gridhelp.Add(btn_help, 0, wx.ALL, 5)
        
        gridexit = wx.GridSizer(1, 3, 0, 0)#buttons
        gridexit.Add(self.btn_cancel, 0, wx.ALL, 5)
        gridexit.Add(self.btn_ok, 0, wx.ALL , 5)
        gridexit.Add(btn_reset, 0, wx.ALL , 5)
        
        gridBtn = wx.GridSizer(1, 2, 0, 0)#buttons
        gridBtn.Add(gridhelp)
        gridBtn.Add(gridexit)
        sizerBase.Add(gridBtn,1, wx.ALL|wx.ALIGN_CENTRE, 10)
        self.SetSizer(sizerBase)
        sizerBase.Fit(self)
        self.Layout()
        
        """--------------------Binders (EVT)----------------------"""
        self.Bind(wx.EVT_BUTTON, self.on_help, btn_help)
        self.Bind(wx.EVT_BUTTON, self.on_cancel, self.btn_cancel)
        self.Bind(wx.EVT_BUTTON, self.on_apply, self.btn_ok)
        self.Bind(wx.EVT_BUTTON, self.on_reset, btn_reset)
        
    #------------------------------------------------------------------#
    def on_help(self, event):
        """
        """
        page = ('https://jeanslack.github.io/Videomass/Pages/'
                'Audio_Parameters/Audio_parameters.html')
        webbrowser.open(page)
    
    #------------------------------------------------------------------#
    def on_reset(self, event):
        """
        Reset all option and values
        """
        self.rdb_sample_r.SetSelection(0)
        self.rdb_bitdepth.SetSelection(0)
        self.rdb_bitrate.SetSelection(0)
        self.rdb_channels.SetSelection(0)

    #------------------------------------------------------------------#
    def on_cancel(self, event):
        """
        if you enable self.Destroy(), it delete from memory all data event and
        no return correctly. It has the right behavior if not used here, because 
        it is called in the main frame. 
        
        Event.Skip(), work correctly here.
        
        """
        #self.Destroy()
        event.Skip()
        
    #------------------------------------------------------------------#
    def on_apply(self, event):
        """
        if you enable self.Destroy(), it delete from memory all data event and
        no return correctly. It has the right behavior if not used here, because 
        it is called in the main frame. 
        
        Event.Skip(), work correctly here. Sometimes needs to disable it for
        needs to maintain the view of the window (for exemple).
        
        """
        self.GetValue()
        #self.Destroy()
        event.Skip()
        
    #------------------------------------------------------------------#
    def GetValue(self):
        """
        This method return values via the interface GetValue()
        """
        for k,v in self.channels.items():
            if self.rdb_channels.GetStringSelection() in v[0]:
                channel = v
                
        for k,v in self.sample_rate.items():
            if self.rdb_sample_r.GetStringSelection() in v[0]:
                samplerate = v
                
        for k,v in self.bitrate.items():
            if self.rdb_bitrate.GetStringSelection() in v[0]:
                bitrate = v
                
        for k,v in self.bitdepth.items():
            if self.rdb_bitdepth.GetStringSelection() in v[0]:
                bitdepth = v

        return (channel, samplerate, bitrate, bitdepth)
    
#######################################################################
class TypeAudioParameters(object):
    """
    The class provides an adequate representation of the different 
    audio parameters that need to be encoded or decoded by FFmpeg. 
    These parameters relate to some aspects of quality and technical: 
    audio bitrates, sample rate, audio channels and bitdepth and also 
    include messages tooltip.
    """
    channel_tooltip = (_("""\
The audio channels are represented by monophonic, 
stereophonic and quadraphonic techniques reproduction.
For some codecs can only assign an audio stream
monaural or stereo, for others even polyphonic.
If you are not sure set to "Default" and source values 
will be copied.\
"""))
    sample_rate_tooltip = (_("""\
The audio Rate (or sample-rate) is the sound sampling 
frequency and is measured in hertz. The higher the frequency, 
plus the audio signal will be true to the sound source, but 
the file will increase its size. For normal playback with 
audio CD set a sample rate of 44100kHz. If you are not sure 
set to "Default" and source values will be copied.\
"""))
    bitrate_tooltip = (_("""\
The audio bitrate affects on file compression
and on the quality of listening. The higher
the value and more higher quality.\
"""))
    bitdepth_tooltip = (_("""\
bit depth is the number of bits of information in each 
sample, and it directly corresponds to the resolution 
of each sample. Bit depth is only meaningful in reference 
to a PCM digital signal. Non-PCM formats, such as lossy 
compression formats, do not have associated bit depths.\
"""))
    def __init__(self, audio_format):
        """
        Accept a type string object representing the name of the audio 
        format. For now there is support for these  audio formats: 
        wav, aiff, flac, alac, aac, ac3, ogg, mp3.
        Each attribute is instantiable with this class and returns the 
        data object for each dictionary.
        """
        self.sample_rate = {0:("Default", ""), 
                            1:("44100 Hz ","-ar 44100 "), 
                            2:("48000 Hz ","-ar 48000"), 
                            3:("88200 Hz ","-ar 88200"), 
                            4:("96000 Hz ","-ar 96000 ")
                            }
        self.channels = None
        self.bitrate = None
        self.bitdepth = None

        if audio_format in ('wav','aiff'):
            self.wav_param()
        elif audio_format in ('flac'):
            self.flac_param()
        elif audio_format in ('alac'):
            self.alac_param()
        elif audio_format == 'aac':
            self.aac_param()
        elif audio_format == 'ac3':
            self.ac3_param()
        elif audio_format == 'ogg':
            self.ogg_param()
        elif audio_format == 'mp3':
            self.mp3_param()
    #-----------------------------------------------------------------#
    def wav_param(self):
        """
        NOTE: the wav and aiff bitdepth is used impicitly on the 
              codec name and not as separated -sample_fmts option.
        """
        self.sample_rate
        self.channels = {0:("Default",""), 1:("Mono","-ac 1"), 
                         2:("Stereo","-ac 2")
                         }
        self.bitdepth = {0:("Default",""),1:("16 bit","-c:a pcm_s16le"),
                         2:("24 bit","-c:a pcm_s24le"),
                         4:("32 bit","-c:a pcm_s32le")
                         }
    #-----------------------------------------------------------------#
    def flac_param(self):
        """
        """
        self.sample_rate
        self.channels = {0:("Default",""), 1:("Mono","-ac 1"), 
                         2:("Stereo","-ac 2")
                         }
        self.bitrate = {0:("Default",""), 
                        1:("very high quality", "-compression_level 0"), 
                        2:("quality 1", "-compression_level 1"), 
                        3:("quality 2", "-compression_level 2"), 
                        4:("quality 3", "-compression_level 3"), 
                        5:("quality 4", "-compression_level 4"), 
                        6:("Standard quality", "-compression_level 5"), 
                        7:("quality 6", "-compression_level 6"), 
                        8:("quality 7", "-compression_level 7"), 
                        9:("low quality", "-compression_level 8")
                        }
        #self.bitdepth = {0:("Default",""),1:("16 bit","-sample_fmt s16"),
                         #2:("24 bit","-sample_fmt s24"),
                         #4:("32 bit","-sample_fmt s32")
                         #} 
    #-----------------------------------------------------------------#
    def alac_param(self):
        """
        """
        self.sample_rate 
        self.channels = {0:("Default",""), 1:("Mono","-ac 1"), 
                         2:("Stereo","-ac 2")
                         }
        
    #-----------------------------------------------------------------#
    def aac_param(self):
        """
        """
        self.sample_rate 
        self.channels = {0:("Default",""), 
                         1:("Mono","-ac 1"), 
                         2:("Stereo","-ac 2"), 
                         3:("MultiChannel 5.1", "-ac 6")
                         }
        self.bitrate = {
            0:("Default",""), 
            1:("low quality", "-b:a 128k"), 
            2:("medium/low quality", "-b:a 160k"), 
            3:("medium quality", "-b:a 192k"), 
            4:("good quality", "-b:a 260k"), 
            5:("very good quality", "-b:a 320k")
            }

    #-----------------------------------------------------------------#
        
    def ac3_param(self):
        """
        """
        self.sample_rate 
        self.channels = {0:("Default",""), 
                         1:("Mono","-ac 1"), 
                         2:("Stereo","-ac 2"), 
                         3:("MultiChannel 5.1", "-ac 6")
                         }
        self.bitrate = {0:("Default",""), 
                        1:("low quality", "-b:a 192k"), 
                        2:("224 kbit/s", "-b:a 224k"), 
                        3:("256 kbit/s", "-b:a 256k"), 
                        4:("320 kbit/s", "-b:a 320k"), 
                        5:("384 kbit/s", "-b:a 384k"), 
                        6:("448 kbit/s", "-b:a 448k"), 
                        7:("512 kbit/s", "-b:a 512k"), 
                        8:("576 kbit/s", "-b:a 576k"), 
                        9:("very good quality", "-b:a 640k")
                        }

    #-----------------------------------------------------------------#
    def ogg_param(self):
        """
        """
        self.sample_rate 
        
        self.channels = {0:("Default",""), 1:("Mono","-ac 1"), 
                         2:("Stereo","-ac 2")
                         }
        self.bitrate = {0:("Default", ""), 
                        1:("very poor quality", "-aq 1"), 
                        2:("VBR 92 kbit/s", "-aq 2"), 
                        3:("VBR 128 kbit/s", "-aq 3"), 
                        4:("VBR 160 kbit/s", "-aq 4"), 
                        5:("VBR 175 kbit/s", "-aq 5"), 
                        6:("VBR 192 kbit/s", "-aq 6"), 
                        7:("VBR 220 kbit/s", "-aq 7"), 
                        8:("VBR 260 kbit/s", "-aq 8"), 
                        9:("VBR 320 kbit/s", "-aq 9"), 
                        10:("very good quality", "-aq 10")
                        }

    #-----------------------------------------------------------------#
    def mp3_param(self):
        """
        """
        self.sample_rate 
        
        self.channels = {0:("Default",""), 1:("Mono","-ac 1"), 
                         2:("Stereo","-ac 2")
                         }
        self.bitrate = {
            0:("Default", ""), 
            1:("VBR 128 kbit/s (low quality)", "-b:a 128k"), 
            2:("VBR 160 kbit/s", "-b:a 160k"), 
            3:("VBR 192 kbit/s", "-b:a 192k"), 
            4:("VBR 260 kbit/s", "-b:a 260k"), 
            5:("CBR 320 kbit/s (very good quality)", "-b:a 320k")
                        }

    #-----------------------------------------------------------------#



# -*- coding: UTF-8 -*-

#########################################################
# Name: mediainfo.py
# Porpose: show ffprobe info for media files
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev (06) December 28 2018
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

import wx
import os
import webbrowser
from videomass3.vdms_IO.IO_tools import FFProbe

class Mediainfo(wx.Dialog):
    """
    Show dialog for display tag and metadata info. 
    """
    def __init__(self, title, path, ffprobe_link, OS):
        # with 'None' not depend from videomass. With 'parent, -1' if close
        # videomass also close mediainfo window:
        #wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)
        wx.Dialog.__init__(self, None, style=wx.DEFAULT_DIALOG_STYLE)
        
        notebook_1 = wx.Notebook(self, wx.ID_ANY)
        notebook_1_pane_1 = wx.Panel(notebook_1, wx.ID_ANY)
        format_info = wx.ListCtrl(notebook_1_pane_1, wx.ID_ANY,
                                  style=wx.LC_REPORT | wx.SUNKEN_BORDER
                                  )
        notebook_1_pane_2 = wx.Panel(notebook_1, wx.ID_ANY)
        
        streams_info = wx.ListCtrl(notebook_1_pane_2, wx.ID_ANY,
                                   style=wx.LC_REPORT | wx.SUNKEN_BORDER
                                   )
        #button_help = wx.Button(self, wx.ID_HELP, "")
        button_close = wx.Button(self, wx.ID_CLOSE, "")
        
        #----------------------Properties----------------------#
        self.SetTitle(title)
        format_info.SetMinSize((640, 300))
        #format_info.SetBackgroundColour(wx.Colour(217, 255, 255))
        format_info.InsertColumn(0, _('names'), width=200)
        format_info.InsertColumn(1, _('parameters'), width=450)
        streams_info.SetMinSize((640, 300))
        #streams_info.SetBackgroundColour(wx.Colour(217, 255, 255))
        streams_info.InsertColumn(0, _('names'), width=200)
        streams_info.InsertColumn(1, _('parameters'), width=450)
        if OS == 'Darwin':
            format_info.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
            streams_info.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
        else:
            streams_info.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
            format_info.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
        #----------------------Layout--------------------------#
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        grid_sizer_1 = wx.FlexGridSizer(2, 1, 0, 0)
        grid_buttons = wx.FlexGridSizer(1, 1, 0, 0)
        sizer_tab2 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab1 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab1.Add(format_info, 1, wx.ALL | wx.EXPAND, 5)
        notebook_1_pane_1.SetSizer(sizer_tab1)
        sizer_tab2.Add(streams_info, 1, wx.ALL | wx.EXPAND, 5)
        notebook_1_pane_2.SetSizer(sizer_tab2)
        notebook_1.AddPage(notebook_1_pane_1, (_("Format overview")))
        notebook_1.AddPage(notebook_1_pane_2, (_("All format streams")))
        grid_sizer_1.Add(notebook_1, 1, wx.ALL|wx.EXPAND, 5)
        grid_buttons.Add(button_close, 0, wx.ALL, 5)
        grid_sizer_1.Add(grid_buttons, flag=wx.ALIGN_RIGHT|wx.RIGHT, border=0)
        #grid_sizer_1.AddGrowableRow(0)
        #grid_sizer_1.AddGrowableRow(1)
        #grid_sizer_1.AddGrowableCol(0)
        sizer_1.Add(grid_sizer_1, 1, wx.EXPAND, 0)
        self.SetSizer(sizer_1)
        sizer_1.Fit(self)
        self.Layout()
        
        # delete previous append:
        format_info.DeleteAllItems()
        streams_info.DeleteAllItems()
        # create instance FFProbe class:
        metadata = FFProbe(path, ffprobe_link, 'pretty') 
        # execute a control for errors:
        if metadata.ERROR():
            wx.MessageBox("[FFprobe] Error:  %s" % (metadata.error), 
                    "Videomass: FFprobe", wx.ICON_ERROR, self)
            self.Destroy()
            return
        
        # create methods instances:
        video_list = metadata.video_stream()
        format_list = metadata.data_format()
        audio_list = metadata.audio_stream()
        subtitle_list = metadata.subtitle_stream()
                
        #populate format_info listctrl output:
        index = 0
        if format_list == []:
            print ('No FORMAT stream metadata found')
        else:
            n = len(format_list)
            for a in range(n):
                (key, value) = format_list[a][0].strip().split('=')
                num_items = format_info.GetItemCount()
                format_info.InsertItem(num_items, 'FORMAT STREAM:')
                format_info.SetItemBackgroundColour(index, "green")
                index +=1
                for b in format_list[a]:
                    (key, value) = b.strip().split('=',1)
                    format_info.InsertItem(index, key)
                    format_info.SetItem(index, 1, value)
                    index += 1
        
        #populate stream_info listctrl output:
        index = 0 
        if video_list == []:
            print ('No VIDEO stream metadata found')
        else:
            n = len(video_list)
            for a in range(n):
                (key, value) = video_list[a][0].strip().split('=')
                num_items = streams_info.GetItemCount()
                streams_info.InsertItem(num_items, 
                               'VIDEO STREAM (index %s):' % (value[0]))
                streams_info.SetItemBackgroundColour(index, "green")
                index +=1
                for b in video_list[a]:
                    (key, value) = b.strip().split('=',1)
                    streams_info.InsertItem(index, key)
                    streams_info.SetItem(index, 1, value)
                    index += 1
                    
        if audio_list == []:
            print ('No AUDIO stream metadata found')
        else:    
            n = len(audio_list)
            for a in range(n):
                (key, value) = audio_list[a][0].strip().split('=')
                num_items = streams_info.GetItemCount()
                streams_info.InsertItem(num_items, 
                               'AUDIO STREAM (index %s):' % (value[0]))
                streams_info.SetItemBackgroundColour(index, "green")
                index +=1
                for b in audio_list[a]:
                    (key, value) = b.strip().split('=',1)
                    streams_info.InsertItem(index, key)
                    streams_info.SetItem(index, 1, value)
                    index += 1
                    
        if subtitle_list == []:
            print ('No SUBTITLE stream metadata found')
        else:
            n = len(subtitle_list)
            for a in range(n):
                (key, value) = subtitle_list[a][0].strip().split('=')
                num_items = streams_info.GetItemCount()
                streams_info.InsertItem(num_items, 
                            'SUBTITLE STREAM (index %s):' % (value[0]))
                streams_info.SetItemBackgroundColour(index, "green")
                index +=1
                for b in subtitle_list[a]:
                    (key, value) = b.strip().split('=',1)
                    streams_info.InsertItem(index, key)
                    streams_info.SetItem(index, 1, value)
                    index += 1
                    
        #----------------------Binding (EVT)----------------------#
        self.Bind(wx.EVT_BUTTON, self.on_close, button_close)
        self.Bind(wx.EVT_CLOSE, self.on_close) # controlla la chiusura (x)
        #self.Bind(wx.EVT_BUTTON, self.on_help, button_help)

    #----------------------Event handler (callback)----------------------#
    def on_close(self, event):
        self.Destroy()
        #event.Skip()

    #-------------------------------------------------------------------#
        



# -*- coding: UTF-8 -*-

#########################################################
# Name: settings.py
# Porpose: videomass setup dialog
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev (06) December 27 2018
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

import wx
import os
import webbrowser

class Setup(wx.Dialog):
    """
    Main settings of the videomass program and configuration storing.
    """
    def __init__(self, parent, threads, cpu_used, ffplay_loglev, ffmpeg_loglev, 
                 ffmpeg_link, ffmpeg_check, ffprobe_link, ffprobe_check, 
                 ffplay_link, ffplay_check, OS, iconset, fileconf, PWD
                 ):
        """
        NOTE 0): self.rowsNum attribute is a sorted list with a exatly number 
                 index corresponding to each read line of the videomass.conf.
        NOTE 1): The code block (USEFUL FOR DEBUGGING) prints in console a 
                 convenient representation of the parsing, which can also be 
                 efforting consulted for future implementations.
                 - POSITION, the number index of self.rowsNum items (how many 
                   objects it contains).
                 - ROW, is the current numeric rows on the file configuration
                 - VALUE, is the value as writing in the file configuration
        """
        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)
        """constructor"""
        
        # Make a items list of
        self.rowsNum = []#rows number list
        dic = {} # used for debug
        with open (fileconf, 'r') as f:
            self.full_list = f.readlines()
        for a,b in enumerate(self.full_list):
            if not b.startswith('#'):
                if not b == '\n':
                    self.rowsNum.append(a)
                    #dic [a] = b.strip()# used for easy reading print debug

        ##USEFUL FOR DEBUGGING (see Setup.__init__.__doc__)
        ##uncomment the following code for a convenient reading
        #print("\nPOSITION:    ROW:     VALUE:")
        #for n, k in enumerate(sorted(dic)):
            #print(n, ' -------> ', k, ' --> ', dic[k])

        self.threads = threads
        self.cpu_used = cpu_used
        self.ffplay_loglevel = ffplay_loglev
        self.ffmpeg_loglevel = ffmpeg_loglev
        self.ffmpeg_link = ffmpeg_link
        self.ffmpeg_check = ffmpeg_check
        self.ffprobe_link = ffprobe_link
        self.ffprobe_check = ffprobe_check
        self.ffplay_link = ffplay_link
        self.ffplay_check = ffplay_check
        self.OS = OS
        self.iconset = iconset
        self.FILEconf = fileconf
        self.PWD = PWD
        
        if self.OS == 'Windows':
            self.ffmpeg = 'ffmpeg.exe'
            self.ffprobe = 'ffprobe.exe'
            self.ffplay = 'ffplay.exe'
        else:
            self.ffmpeg = 'ffmpeg'
            self.ffprobe = 'ffprobe'
            self.ffplay = 'ffplay'
        #----------------------------set notebook and tabs pages
        notebook = wx.Notebook(self, wx.ID_ANY, style=0)
        
        tabOne = wx.Panel(notebook, wx.ID_ANY)
        notebook.AddPage(tabOne, _("General"))
        
        tabTwo = wx.Panel(notebook, wx.ID_ANY)
        notebook.AddPage(tabTwo, _("Logging levels"))
        
        tabThree = wx.Panel(notebook, wx.ID_ANY)
        notebook.AddPage(tabThree, _("Executable paths"))
        
        tabFour = wx.Panel(notebook, wx.ID_ANY)
        notebook.AddPage(tabFour, _("Appearance"))
        # make a sizer base and grid base. Note that grid base contains
        # buttons close and apply. First add notebook then the buttons at bottom
        sizer = wx.BoxSizer(wx.VERTICAL)
        gridBase = wx.FlexGridSizer(2, 1, 0, 0)# ntbook + buttons ok,close
        gridBase.Add(notebook, 1, wx.ALL|wx.EXPAND, 15)
        #--------------------------------------------------TAB 1
        gridGeneral = wx.FlexGridSizer(3, 1, 0, 0)
        tabOne.SetSizer(gridGeneral)#aggiungo il sizer su tab 1
        boxLabThreads = wx.StaticBoxSizer(wx.StaticBox(tabOne, wx.ID_ANY, (
                                    _("Settings CPU"))), wx.VERTICAL)
        gridGeneral.Add(boxLabThreads, 1, wx.ALL|wx.EXPAND, 15)
        gridThreads = wx.FlexGridSizer(4, 1, 0, 0)
        boxLabThreads.Add(gridThreads, 1, wx.ALL|wx.EXPAND, 15)
        lab1_pane1 = wx.StaticText(tabOne, wx.ID_ANY,(
                               _("Set the number of threads (from 0 to 32)")))
        gridThreads.Add(lab1_pane1, 0, wx.ALL, 5)
        self.spinctrl_threads = wx.SpinCtrl(tabOne, wx.ID_ANY, 
                                            "%s" % threads[9:],
                                            size=(-1,-1), min=0, max=32, 
                                            style=wx.TE_PROCESS_ENTER
                                             )
        gridThreads.Add(self.spinctrl_threads, 0, wx.ALL |
                                                  wx.ALIGN_CENTER_VERTICAL, 
                                                  5)
        lab2_pane1 = wx.StaticText(tabOne, wx.ID_ANY, (
                        _("Quality/Speed ratio modifier (from -16 to 16)")))
        gridThreads.Add(lab2_pane1, 0, wx.ALL, 5)
        gridctrl = wx.FlexGridSizer(1, 2, 0, 0)
        gridThreads.Add(gridctrl)
        self.spinctrl_cpu = wx.SpinCtrl(tabOne, wx.ID_ANY, 
                                        "%s" % cpu_used[9:], min=-16, max=16, 
                                        size=(-1,-1), style=wx.TE_PROCESS_ENTER
                                             )
        gridctrl.Add(self.spinctrl_cpu, 0, wx.ALL, 5)
        self.ckbx_cpu = wx.CheckBox(tabOne, wx.ID_ANY, (_("Disable")))
        gridctrl.Add(self.ckbx_cpu, 1, wx.ALL, border=10 )
        #gridctrl.Add(self.ckbx_autoThreads, 0,  wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5)
        
        #--------------------------------------------------TAB 2
        gridLog = wx.FlexGridSizer(4, 1, 0, 0)
        tabTwo.SetSizer(gridLog)#aggiungo il sizer su tab 2
        lab3_pane2 = wx.StaticText(tabTwo, wx.ID_ANY, 
                               (_("These settings affect output messages "
                                  "and the log\nmessages during processes. "
                                  "Change only if you\nknow what you are "
                                  "doing.")))
        gridLog.Add(lab3_pane2, 0, wx.ALL, 15)
        self.rdbFFmpeg = wx.RadioBox(tabTwo, wx.ID_ANY,
                                    ("Set logging level flags used by FFmpeg"), 
                            choices=[("quiet (Show nothing at all)"), 
                                    ("fatal (Only show fatal errors)"), 
                                    ("error (Show all errors)"), 
                                    ("warning (Show all warnings and errors)"), 
                                    ("info (Show informative messages "
                                            "during processing)"),], 
                                    majorDimension=1, style=wx.RA_SPECIFY_COLS
                                    )
        gridLog.Add(self.rdbFFmpeg, 0, wx.ALL, 15)
        self.rdbFFplay = wx.RadioBox(tabTwo, wx.ID_ANY,
                                    ("Set logging level flags used by FFplay"), 
                            choices=[("quiet (Show nothing at all)"), 
                                    ("fatal (Only show fatal errors)"), 
                                    ("error (Show all errors)"), 
                                    ("warning (Show all warnings and errors)"), 
                                    ("info (Show informative messages "
                                            "during processing)"),], 
                                    majorDimension=1, style=wx.RA_SPECIFY_COLS
                                    )
        gridLog.Add(self.rdbFFplay, 0, wx.ALL, 15)
        #--------------------------------------------------TAB 3
        gridExec = wx.FlexGridSizer(6, 1, 0, 0)
        tabThree.SetSizer(gridExec)#aggiungo il sizer su tab 3

        self.checkbox_exeFFmpeg = wx.CheckBox(tabThree, wx.ID_ANY,(
                                       _(" Use a custom path to run FFmpeg")))
        self.btn_pathFFmpeg = wx.Button(tabThree, wx.ID_ANY, _("Browse.."))
        self.txtctrl_ffmpeg = wx.TextCtrl(tabThree, wx.ID_ANY, "",
                                          style=wx.TE_READONLY
                                          )
        gridExec.Add(self.checkbox_exeFFmpeg, 1, wx.TOP|
                                                 wx.ALIGN_CENTER_VERTICAL|
                                                 wx.ALIGN_CENTER_HORIZONTAL, 15)
        gridFFmpeg = wx.FlexGridSizer(1, 2, 0, 0)
        gridExec.Add(gridFFmpeg)
        gridFFmpeg.Add(self.btn_pathFFmpeg, 0, wx.ALL, 15)
        gridFFmpeg.Add(self.txtctrl_ffmpeg, 0, wx.ALIGN_CENTER_VERTICAL, 5)

        self.checkbox_exeFFprobe = wx.CheckBox(tabThree, wx.ID_ANY, (
                                       _(" Use a custom path to run FFprobe")))
        self.btn_pathFFprobe = wx.Button(tabThree, wx.ID_ANY, _("Browse.."))
        self.txtctrl_ffprobe = wx.TextCtrl(tabThree, wx.ID_ANY, "", 
                                           style=wx.TE_READONLY
                                           )
        gridExec.Add(self.checkbox_exeFFprobe, 1, wx.TOP|
                                                  wx.ALIGN_CENTER_VERTICAL|
                                                  wx.ALIGN_CENTER_HORIZONTAL, 15)
        gridFFprobe = wx.FlexGridSizer(1, 2, 0, 0)
        gridExec.Add(gridFFprobe)
        gridFFprobe.Add(self.btn_pathFFprobe, 0, wx.ALL, 15)
        gridFFprobe.Add(self.txtctrl_ffprobe, 0, wx.ALIGN_CENTER_VERTICAL, 5)

        self.checkbox_exeFFplay = wx.CheckBox(tabThree, wx.ID_ANY, (
                                       _(" Use a custom path to run FFplay")))
        self.btn_pathFFplay = wx.Button(tabThree, wx.ID_ANY, _("Browse.."))
        self.txtctrl_ffplay = wx.TextCtrl(tabThree, wx.ID_ANY, "", 
                                          style=wx.TE_READONLY
                                          )
        gridExec.Add(self.checkbox_exeFFplay, 1, wx.TOP|
                                                 wx.ALIGN_CENTER_VERTICAL|
                                                 wx.ALIGN_CENTER_HORIZONTAL, 15)
        gridFFplay = wx.FlexGridSizer(1, 2, 0, 0)
        gridExec.Add(gridFFplay)
        gridFFplay.Add(self.btn_pathFFplay, 0, wx.ALL, 15)
        gridFFplay.Add(self.txtctrl_ffplay, 0, wx.ALIGN_CENTER_VERTICAL, 5)
        #--------------------------------------------------TAB 4
        gridappearance = wx.FlexGridSizer(3, 1, 0, 0)
        tabFour.SetSizer(gridappearance)#aggiungo il sizer su tab 4
        boxLabIcons = wx.StaticBoxSizer(wx.StaticBox(tabFour, wx.ID_ANY, (
                                    _("Set Icon Themes"))), wx.VERTICAL)
        gridappearance.Add(boxLabIcons, 1, wx.ALL|wx.EXPAND, 15)
        self.cmbx_icons = wx.ComboBox(tabFour, wx.ID_ANY, 
                         choices=[("Videomass_Sign_Icons"),
                                  ("Material_Design_Icons_black"), 
                                  ("Material_Design_Icons_white"),
                                  ("Flat_Color_Icons"), 
                                  ], style=wx.CB_DROPDOWN | wx.CB_READONLY)
        boxLabIcons.Add(self.cmbx_icons, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 15)
        self.cmbx_icons.SetValue(self.iconset)
        
        boxLabColor = wx.StaticBoxSizer(wx.StaticBox(tabFour, wx.ID_ANY, (
                                    _("Custom Colour UI"))), wx.VERTICAL)
        gridappearance.Add(boxLabColor, 1, wx.ALL|wx.EXPAND, 15)
        gridTBColor = wx.FlexGridSizer(3, 2, 0, 0)
        boxLabColor.Add(gridTBColor)
        
        labTBColor = wx.StaticText(tabFour, wx.ID_ANY, (
                                            _("Change Toolbar Colour:")))
        gridTBColor.Add(labTBColor, 0, wx.ALL | 
                                     wx.ALIGN_CENTER_HORIZONTAL| 
                                     wx.ALIGN_CENTER_VERTICAL, 15)
        
        btn_TBcolor = wx.Button(tabFour, wx.ID_ANY, _("Bar Colour"))
        gridTBColor.Add(btn_TBcolor, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 15)
        
        labTBColorBtn = wx.StaticText(tabFour, wx.ID_ANY, (
                                          _("Change Toolbar Buttons Colour:")))
        gridTBColor.Add(labTBColorBtn, 0, wx.ALL | 
                                     wx.ALIGN_CENTER_HORIZONTAL| 
                                     wx.ALIGN_CENTER_VERTICAL, 15)
        
        btn_TBcolorBtn = wx.Button(tabFour, wx.ID_ANY, _("Buttons Colour"))
        gridTBColor.Add(btn_TBcolorBtn, 0, wx.ALL | wx.ALIGN_CENTER_HORIZONTAL, 15)
        btn_TBcolorClearBtn = wx.Button(tabFour, wx.ID_CLEAR, 
                                        _("Restore default settings"))
        gridappearance.Add(btn_TBcolorClearBtn, 0, wx.ALL | wx.EXPAND | wx.ALIGN_CENTER_HORIZONTAL, 15)

        #------------------------------------------------------bottom
        gridBottom = wx.GridSizer(1, 2, 0, 0)
        btn_help = wx.Button(self, wx.ID_HELP, "")
        gridhelp = wx.GridSizer(1, 1, 0, 0)
        gridhelp.Add(btn_help, 0, wx.ALL, 5)
        gridBottom.Add(gridhelp, 0, wx.ALL, 10)
        btn_close = wx.Button(self, wx.ID_CANCEL, "")
        gridexit = wx.GridSizer(1, 2, 0, 0)
        gridexit.Add(btn_close, 0, wx.ALL, 5)
        btn_ok = wx.Button(self, wx.ID_APPLY, "")
        gridexit.Add(btn_ok, 0, wx.ALL, 5)
        gridBottom.Add(gridexit, 0, wx.ALL, 10)
        gridBase.Add(gridBottom, flag=wx.ALIGN_RIGHT|wx.RIGHT, border=10)
        sizer.Add(gridBase,1, wx.ALL|wx.EXPAND, 5)
        self.SetSizer(sizer)
        sizer.Fit(self)
        self.Layout()
        
        #----------------------Properties----------------------#
        self.SetTitle(_("Videomass: setup"))
        self.spinctrl_cpu.SetToolTip(_("Quality/Speed ratio modifier "
                                            "(from -16 to 16) (default 1)")
                                    )
        
        self.checkbox_exeFFmpeg.SetToolTip(_("Enable custom search for "
                       "the executable FFmpeg. If the checkbox is disabled or "
                       "if the path field is empty, the search of the "
                       "executable is entrusted to the system."))
        
        self.btn_pathFFmpeg.SetToolTip(_("Open path FFmpeg"))
        self.txtctrl_ffmpeg.SetMinSize((200, -1))
        self.txtctrl_ffmpeg.SetToolTip(
                                    _("path to executable binary FFmpeg")
                                                    )
        self.checkbox_exeFFprobe.SetToolTip(_("Path generation file"))
        
        self.checkbox_exeFFmpeg.SetToolTip(_("Enable custom search for "
                    "the executable FFprobe. If the checkbox is disabled or "
                    "if the path field is empty, the search of the "
                    "executable is entrusted to the system."))
        
        self.btn_pathFFprobe.SetToolTip(_("Open path FFprobe"))
        self.txtctrl_ffprobe.SetMinSize((200, -1))
        self.txtctrl_ffprobe.SetToolTip(
                                    _("path to executable binary FFprobe")
                                                    )
        self.checkbox_exeFFplay.SetToolTip(_("Path generation file")
                                                 )
        self.checkbox_exeFFmpeg.SetToolTip(_("Enable custom search for "
                    "the executable FFplay. If the checkbox is disabled or "
                    "if the path field is empty, the search of the "
                    "executable is entrusted to the system.")
                                                  )
        self.btn_pathFFplay.SetToolTip(_("Open path FFplay"))
        self.txtctrl_ffplay.SetMinSize((200, -1))
        self.txtctrl_ffplay.SetToolTip(
                                     _("path to executable binary FFplay"))
        #----------------------Binding (EVT)----------------------#

        self.Bind(wx.EVT_RADIOBOX, self.logging_ffplay, self.rdbFFplay)
        self.Bind(wx.EVT_RADIOBOX, self.logging_ffmpeg, self.rdbFFmpeg)
        self.Bind(wx.EVT_SPINCTRL, self.on_threads, self.spinctrl_threads)
        self.Bind(wx.EVT_SPINCTRL, self.on_cpu_used, self.spinctrl_cpu)
        self.Bind(wx.EVT_CHECKBOX, self.on_cpu_used, self.ckbx_cpu)
        self.Bind(wx.EVT_CHECKBOX, self.exeFFmpeg, self.checkbox_exeFFmpeg)
        self.Bind(wx.EVT_BUTTON, self.open_path_ffmpeg, self.btn_pathFFmpeg)
        self.Bind(wx.EVT_TEXT_ENTER, self.txtffmpeg, self.txtctrl_ffmpeg)
        self.Bind(wx.EVT_CHECKBOX, self.exeFFprobe, self.checkbox_exeFFprobe)
        self.Bind(wx.EVT_BUTTON, self.open_path_ffprobe, self.btn_pathFFprobe)
        self.Bind(wx.EVT_TEXT_ENTER, self.txtffprobe, self.txtctrl_ffprobe)
        self.Bind(wx.EVT_CHECKBOX, self.exeFFplay, self.checkbox_exeFFplay)
        self.Bind(wx.EVT_BUTTON, self.open_path_ffplay, self.btn_pathFFplay)
        self.Bind(wx.EVT_TEXT_ENTER, self.txtffplay, self.txtctrl_ffplay)
        self.Bind(wx.EVT_COMBOBOX, self.on_Iconthemes, self.cmbx_icons)
        
        self.Bind(wx.EVT_BUTTON, self.onColorDlg, btn_TBcolor)
        self.Bind(wx.EVT_BUTTON, self.onColorDlg, btn_TBcolorBtn)
        self.Bind(wx.EVT_BUTTON, self.onAppearanceDefault, btn_TBcolorClearBtn)
        
        self.Bind(wx.EVT_BUTTON, self.on_help, btn_help)
        self.Bind(wx.EVT_BUTTON, self.on_close, btn_close)
        self.Bind(wx.EVT_BUTTON, self.on_ok, btn_ok)

        #--------------------------------------------#
        self.current_settings() # call function for initialize setting layout 

    def current_settings(self):
        """
        Setto l'abilitazione/disabilitazione in funzione del file di conf.
        Setting enable/disable in according to the configuration file
        """
        
        for s in range(self.rdbFFplay.GetCount()):
            if (self.ffplay_loglevel.split()[0] in 
                                self.rdbFFplay.GetString(s).split()[0]
                                ):
                self.rdbFFplay.SetSelection(s)
        
        for s in range(self.rdbFFmpeg.GetCount()):
            if (self.ffmpeg_loglevel.split()[0] in 
                                self.rdbFFmpeg.GetString(s).split()[0]
                                ):
                self.rdbFFmpeg.SetSelection(s)
        
        if self.cpu_used == 'Disabled':
            self.ckbx_cpu.SetValue(True)
            self.spinctrl_cpu.Disable()
        #else:
            #self.ckbx_cpu.SetValue(False)
            #self.spinctrl_cpu.Enable()
            
        if self.ffmpeg_check == 'false':
            self.btn_pathFFmpeg.Disable()
            self.txtctrl_ffmpeg.Disable()
            self.txtctrl_ffmpeg.SetValue("")
            self.checkbox_exeFFmpeg.SetValue(False)
        else:
            self.txtctrl_ffmpeg.AppendText(self.ffmpeg_link)
            self.checkbox_exeFFmpeg.SetValue(True)
            
        if self.ffprobe_check == 'false':
            self.btn_pathFFprobe.Disable()
            self.txtctrl_ffprobe.Disable()
            self.txtctrl_ffprobe.SetValue("")
            self.checkbox_exeFFprobe.SetValue(False)
        else:
            self.txtctrl_ffprobe.AppendText(self.ffprobe_link)
            self.checkbox_exeFFprobe.SetValue(True)
            
        if self.ffplay_check == 'false':
            self.btn_pathFFplay.Disable()
            self.txtctrl_ffplay.Disable()
            self.txtctrl_ffplay.SetValue("")
            self.checkbox_exeFFplay.SetValue(False)
        else:
            self.txtctrl_ffplay.AppendText(self.ffplay_link)
            self.checkbox_exeFFplay.SetValue(True)
    
    #--------------------------------------------------------------------#
    def on_threads(self, event):
        """set cpu number threads used as option on ffmpeg"""
        sett = self.spinctrl_threads.GetValue()
        self.full_list[self.rowsNum[2]] = '-threads %s\n' % sett
    #--------------------------------------------------------------------#
    def on_cpu_used(self, event):
        """set cpu number threads used as option on ffmpeg"""
        if self.ckbx_cpu.IsChecked():
            self.spinctrl_cpu.Disable()
            self.full_list[self.rowsNum[3]] = 'Disabled\n'
        else:
            if not self.spinctrl_cpu.IsEnabled():
                self.spinctrl_cpu.Enable()
            sett = self.spinctrl_cpu.GetValue()
            self.full_list[self.rowsNum[3]] = '-cpu-used %s\n' % sett
    #--------------------------------------------------------------------#
    #--------------------------------------------------------------------#
    def logging_ffplay(self, event):
        """specifies loglevel type for ffplay."""
        s = self.rdbFFplay.GetStringSelection().split()[0]
        self.full_list[self.rowsNum[4]] = '%s -hide_banner\n' % s

    #--------------------------------------------------------------------#
    def logging_ffmpeg(self, event):
        """specifies loglevel type for ffmpeg"""
        s = self.rdbFFmpeg.GetStringSelection().split()[0]
        self.full_list[self.rowsNum[5]] = '%s -stats\n' % s
        
    #----------------------ffmpeg path checkbox--------------------------#
    def exeFFmpeg(self, event):
        """Enable or disable ffmpeg binary esecutable"""
        if self.checkbox_exeFFmpeg.IsChecked():
            self.btn_pathFFmpeg.Enable()
            self.txtctrl_ffmpeg.Enable()
            self.txtctrl_ffmpeg.SetValue("")
            self.full_list[self.rowsNum[6]] = 'true\n'

        else:
            self.btn_pathFFmpeg.Disable()
            self.txtctrl_ffmpeg.Disable()
            self.txtctrl_ffmpeg.SetValue("")
            self.full_list[self.rowsNum[6]] = 'false\n'
            self.full_list[self.rowsNum[7]] = '%s\n' % self.ffmpeg

    #----------------------ffmpeg path open dialog----------------------#
    def open_path_ffmpeg(self, event):
        """Indicates a new ffmpeg path-name"""
        dialogfile = wx.FileDialog(self, 
                _("Videomass: Where is the ffmpeg executable located?"), "", "", 
                "ffmpeg binarys (*%s)|*%s| All files (*.*)|*.*"
                % (self.ffmpeg, self.ffmpeg), 
                wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
            
        if dialogfile.ShowModal() == wx.ID_OK:
            self.txtctrl_ffmpeg.SetValue("")
            self.txtctrl_ffmpeg.AppendText(dialogfile.GetPath())
            self.full_list[self.rowsNum[7]] = '%s\n' % (dialogfile.GetPath())
            dialogfile.Destroy()
    #---------------------------------------------------------------------#
    def txtffmpeg(self, event):
        """write ffmpeg pathname"""
        t = self.txtctrl_ffmpeg.GetValue()
        self.full_list[self.rowsNum[7]] = '%s\n' % (t)

    #----------------------ffprobe path checkbox--------------------------#
    def exeFFprobe(self, event):
        """Enable or disable ffprobe binary esecutable"""
        if self.checkbox_exeFFprobe.IsChecked():
            self.btn_pathFFprobe.Enable()
            self.txtctrl_ffprobe.Enable()
            self.txtctrl_ffprobe.SetValue("")
            self.full_list[self.rowsNum[8]] = 'true\n'

        else:
            self.btn_pathFFprobe.Disable()
            self.txtctrl_ffprobe.Disable()
            self.txtctrl_ffprobe.SetValue("")
            self.full_list[self.rowsNum[8]] = 'false\n'
            self.full_list[self.rowsNum[9]] = '%s\n' % self.ffprobe

    #----------------------ffprobe path open dialog----------------------#
    def open_path_ffprobe(self, event):
        """Indicates a new ffprobe path-name"""
        dialfile = wx.FileDialog(self, 
            _("Videomass: Where is the ffprobe executable located?"), "", "", 
            "ffprobe binarys (*%s)|*%s| All files (*.*)|*.*"
            % (self.ffprobe, self.ffprobe), 
            wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
            
        if dialfile.ShowModal() == wx.ID_OK:
            self.txtctrl_ffprobe.SetValue("")
            self.txtctrl_ffprobe.AppendText(dialfile.GetPath())
            self.full_list[self.rowsNum[9]] = '%s\n' % (dialfile.GetPath())
            dialfile.Destroy()
    #---------------------------------------------------------------------#
    def txtffprobe(self, event):
        """write ffprobe pathname"""
        t = self.txtctrl_ffprobe.GetValue()
        self.full_list[self.rowsNum[9]] = '%s\n' % (t)

    #----------------------ffplay path checkbox--------------------------#
    def exeFFplay(self, event):
        """Enable or disable ffplay binary esecutable"""
        if self.checkbox_exeFFplay.IsChecked():
            self.btn_pathFFplay.Enable()
            self.txtctrl_ffplay.Enable()
            self.txtctrl_ffplay.SetValue("")
            self.full_list[self.rowsNum[10]] = 'true\n'

        else:
            self.btn_pathFFplay.Disable()
            self.txtctrl_ffplay.Disable()
            self.txtctrl_ffplay.SetValue("")
            self.full_list[self.rowsNum[10]] = 'false\n'
            self.full_list[self.rowsNum[11]] = '%s\n' % self.ffplay

    #----------------------ffplay path open dialog----------------------#
    def open_path_ffplay(self, event):
        """Indicates a new ffplay path-name"""
        dialfile = wx.FileDialog(self, 
                _("Videomass: Where is the ffplay executable located?"), "", "", 
                "ffplay binarys (*%s)|*%s| All files (*.*)|*.*"
                % (self.ffplay, self.ffplay), 
                wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
            
        if dialfile.ShowModal() == wx.ID_OK:
            self.txtctrl_ffplay.SetValue("")
            self.txtctrl_ffplay.AppendText(dialfile.GetPath())
            self.full_list[self.rowsNum[11]] = '%s\n' % (dialfile.GetPath())
            
            dialfile.Destroy()
    #---------------------------------------------------------------------#
    def txtffplay(self, event):
        """write ffplay pathname"""
        t = self.txtctrl_ffplay.GetValue()
        self.full_list[self.rowsNum[11]] = '%s\n' % (t)
            
    #--------------------------------------------------------------------#
    def on_Iconthemes(self, event):
        """
        Set themes of icons
        """
        choice = "%s\n" % self.cmbx_icons.GetStringSelection()
        self.full_list[self.rowsNum[12]] = choice
    #------------------------------------------------------------------#
    def onColorDlg(self, event):
        """
        Colorize the toolbar bar
        """
        btn = event.GetEventObject()
        identify = btn.GetLabelText()
  
        dlg = wx.ColourDialog(self)
 
        # Ensure the full colour dialog is displayed, 
        # not the abbreviated version.
        dlg.GetColourData().SetChooseFull(True)
 
        if dlg.ShowModal() == wx.ID_OK:
            data = dlg.GetColourData()
            rgb = str(data.GetColour().Get())
            choice = rgb.replace('(','').replace(')','').strip()
            
            if identify == _('Bar Colour'):
                self.full_list[self.rowsNum[13]] = "%s\n" % choice
            elif identify == _('Buttons Colour'):
                self.full_list[self.rowsNum[14]] = "%s\n" % choice
 
        dlg.Destroy()
    #----------------------------------------------------------------------#
    def onAppearanceDefault(self, event):
        """
        Restore to default settings colors and icons set
        """
        self.full_list[self.rowsNum[12]] = "Videomass_Sign_Icons\n"
        self.full_list[self.rowsNum[13]] = '205,235,222\n'
        self.full_list[self.rowsNum[14]] = '255,255,255\n'
        
    #----------------------------------------------------------------------#
    def on_help(self, event):
        """
        """
        page = 'https://jeanslack.github.io/Videomass/Pages/Startup/Setup.html'
        webbrowser.open(page)
            
    #--------------------------------------------------------------------#
    def on_close(self, event):
        event.Skip()
    #--------------------------------------------------------------------#
    def on_ok(self, event):
        """
        Applies all changes writing the new entries
        """
            
            
        # if self.checkbox_exeFFmpeg.IsChecked():
        #     if not self.txtctrl_ffmpeg.GetValue() == '':
        #         ffmpeg_src = self.txtctrl_ffmpeg.GetValue()
        #         if not self.ffmpeg_link == ffmpeg_src:# if not modified
        #             if os.path.exists("%s/FFMPEG_BIN/bin" % self.PWD):
        #                 os.symlink(ffmpeg_src, "%s/FFMPEG_BIN/bin/ffmpeg" % self.PWD)
        #             
        # if self.checkbox_exeFFprobe.IsChecked():
        #     if not self.txtctrl_ffprobe.GetValue() == '':
        #         ffprobe_src = self.txtctrl_ffprobe.GetValue()
        #         if not self.ffprobe_link == ffprobe_src:# if not modified
        #             if os.path.exists("%s/FFMPEG_BIN" % self.PWD):
        #                 os.symlink(ffprobe_src, "%s/FFMPEG_BIN/bin/ffprobe" % self.PWD)
        # 
        # if self.checkbox_exeFFplay.IsChecked():
        #     if not self.txtctrl_ffplay.GetValue() == '':
        #         ffplay_src = self.txtctrl_ffplay.GetValue()
        #         if not self.ffplay_link == ffplay_src:# if not modified
        #             if os.path.exists("%s/FFMPEG_BIN" % self.PWD):
        #                 os.symlink(ffplay_src, "%s/FFMPEG_BIN/bin/ffplay" % self.PWD)

        with open (self.FILEconf, 'w') as fileconf:
            for i in self.full_list:
                fileconf.write('%s' % i)
        wx.MessageBox(_("Changes will take affect once the program " 
                        "has been restarted"))
        
        #self.Destroy() # WARNING on mac not close corretly, on linux ok
        self.Close()
        event.Skip()
        



# -*- coding: UTF-8 -*-

#########################################################
# Name: dragNdrop.py
# Porpose: drag n drop interface
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: Dec.28.2018, Sept.12.2019
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

import wx
import os
from videomass3.vdms_IO import IO_tools

dirname = os.path.expanduser('~') # /home/user/
duration = []
azure = '#d9ffff' # rgb form (wx.Colour(217,255,255))
red = '#ea312d'
yellow = '#a29500'
greenolive = '#6aaf23'
orange = '#f28924'

########################################################################
class MyListCtrl(wx.ListCtrl):
    """
    This is the listControl widget. Note that this wideget has DnDPanel
    parent.
    """
    #----------------------------------------------------------------------
    def __init__(self, parent, fileList, ffprobe_link):
        """Constructor"""
        wx.ListCtrl.__init__(self, parent, style=wx.LC_REPORT|wx.LC_SINGLE_SEL)
        self.index = 0
        self.parent = parent # parent is DnDPanel class
        self.fileList = fileList
        self.invalid = False # used for directories import control
        self.ffprobe_link = ffprobe_link
        

    #----------------------------------------------------------------------#
    def dropUpdate(self, path):
        """
        Update list-control and fileList during drag and drop.
        Also, enable certain buttons of the MainFrame toolbar
        and advise with status bar messages
        """
        if os.path.isdir(path):
            mess = _("Directories/folders are not accepted: > '%s'") % path
            print (mess)
            self.parent.statusbar_msg(mess, orange)
            self.invalid = True
            return
        
        if path not in self.fileList:
            self.invalid = False
            self.fileList.append(path)
            self.InsertItem(self.index, path)
            self.index += 1
            s = IO_tools.probeDuration(path, self.ffprobe_link)
            duration.append(s[0])
            if s[1]:
                if s[1] == 'N/A':
                    msg = "%s; %s" %(s[1],_('duration is skipped'))
                    self.parent.statusbar_msg(msg, greenolive)
                else:
                    self.parent.statusbar_msg(s[1], red)
        else:
            mess = _("Duplicate files are not accepted: > '%s'") % path
            print (mess)
            self.parent.statusbar_msg(mess, yellow)
            
    #----------------------------------------------------------------------#
    def send_data(self):
        """
        enable buttons and send data to parent in non-recursive mode
        """
        self.parent.btn_enable(self.fileList, self.invalid)

########################################################################
class MyFileDropTarget(wx.FileDropTarget):
    """
    This is the file drop target
    """
    #----------------------------------------------------------------------#
    def __init__(self, window):
        """
        Constructor. File Drop targets are subsets of windows
        """
        wx.FileDropTarget.__init__(self)
        self.window = window # window is MyListCtr class
    #----------------------------------------------------------------------#
    def OnDropFiles(self, x, y, filenames):
        """
        When files are dropped, write where they were dropped and then
        the file paths themselves
        """
        for filepath in filenames:
            self.window.dropUpdate(filepath) # update list control

        # send data at end of every drag n drop action:
        self.window.send_data()
        return True

########################################################################
class DnDPanel(wx.Panel):
    """
    Panel for dragNdrop files queue. Accept one or more files.
    """
    def __init__(self, parent, ffprobe_link):  
        """Constructor. This will initiate with an id and a title"""
        wx.Panel.__init__(self, parent=parent)
        self.parent = parent # parent is the MainFrame
        self.file_dest = dirname # path name files destination
        self.fileList = [] # list of each imported file
        self.duration = duration # list of the duration of each file imported
        self.switch = 'off' # tells if one or more files are imported
        self.selected = False # tells if an imported file is selected or not
        #This builds the list control box:
        self.fileListCtrl = MyListCtrl(self, self.fileList, ffprobe_link)  #class MyListCtr
        #Establish the listctrl as a drop target:
        file_drop_target = MyFileDropTarget(self.fileListCtrl)
        self.fileListCtrl.SetDropTarget(file_drop_target) #Make drop target.

        # create widgets
        btn_clear = wx.Button(self, wx.ID_CLEAR, "")
        self.ckbx_dir = wx.CheckBox(self, wx.ID_ANY, (
                                _("Save destination in source folder")))
        self.btn_save = wx.Button(self, wx.ID_OPEN, "...", size=(-1,-1))
        self.text_path_save = wx.TextCtrl(self, wx.ID_ANY, "", 
                                    style=wx.TE_PROCESS_ENTER | wx.TE_READONLY
                                                    )
        lbl = wx.StaticText(self, label=_("Drag some files here:"))
        self.fileListCtrl.InsertColumn(0,_('Files list'),width=700)
        # create sizers layout
        sizer = wx.BoxSizer(wx.VERTICAL)
        grid = wx.FlexGridSizer(1, 4, 5, 5)
        sizer.Add(lbl, 0, wx.ALL, 5)
        sizer.Add(self.fileListCtrl, 1, wx.EXPAND|wx.ALL, 5)
        sizer.Add(grid)
        grid.Add(btn_clear, 1, wx.ALL, 5)
        grid.Add(self.ckbx_dir, 1, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|
                 wx.ALIGN_CENTER_VERTICAL, 5
                 )
        grid.Add(self.btn_save, 1, wx.ALL, 5)
        grid.Add(self.text_path_save, 1, wx.ALL|wx.EXPAND, 5)
        self.text_path_save.SetMinSize((290, -1))
        self.SetSizer(sizer)
        
        self.Bind(wx.EVT_BUTTON, self.deleteAll, btn_clear)
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_select, self.fileListCtrl)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.on_deselect, self.fileListCtrl)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.on_doubleClick, self.fileListCtrl)
        self.Bind(wx.EVT_CONTEXT_MENU, self.onContext)
        #self.Bind(wx.EVT_CHECKBOX, self.same_filedest, self.ckbx_dir)
        #self.Bind(wx.EVT_BUTTON, self.on_custom_save, self.btn_save)
        
        self.text_path_save.SetValue(self.file_dest)
        
    #----------------------------------------------------------------------
    def onContext(self, event):
        """
        Create and show a Context Menu
        """
        # only do this part the first time so the events are only bound once 
        if not hasattr(self, "popupID1"):
            self.popupID1 = wx.NewId()
            self.itemTwoId = wx.NewId()
            self.itemThreeId = wx.NewId()
            self.Bind(wx.EVT_MENU, self.onPopup, id=self.popupID1)
            self.Bind(wx.EVT_MENU, self.onPopup, id=self.itemTwoId)
            self.Bind(wx.EVT_MENU, self.onPopup, id=self.itemThreeId)
 
        # build the menu
        menu = wx.Menu()
        itemOne = menu.Append(self.popupID1, _("Play selected file"))
        itemTwo = menu.Append(self.itemTwoId, _("Show metadata window"))
        itemThree = menu.Append(self.itemThreeId, _("Remove the selected file"))
 
        # show the popup menu
        self.PopupMenu(menu)
        menu.Destroy()
    #----------------------------------------------------------------------
    def onPopup(self, event):
        """
        Evaluate the label string of the menu item selected and start
        the related process
        """
        itemId = event.GetId()
        menu = event.GetEventObject()
        menuItem = menu.FindItemById(itemId)
        

        if not self.selected:
            self.parent.statusbar_msg(_('No file selected to `%s` yet') % 
                                      menuItem.GetLabel(), yellow)
        else:
            self.parent.statusbar_msg('Add Files', None)
            if menuItem.GetLabel() == _("Play selected file"):
                self.parent.ImportPlay()
                
            elif menuItem.GetLabel() == _("Show metadata window"):
                #self.on_doubleClick(self)
                self.parent.ImportInfo(self)
                
            elif menuItem.GetLabel() == _("Remove the selected file"):
                if self.fileListCtrl.GetItemCount() == 1:
                    self.deleteAll(self)
                else:
                    item = self.fileListCtrl.GetFocusedItem()
                    self.fileListCtrl.DeleteItem(item)
                    self.parent.importClicked_disable()
                    self.selected = False
                    self.fileList.pop(item)
                    self.duration.pop(item)
                
    #----------------------------------------------------------------------
    def btn_enable(self, fileList, invalid):
        """
        When is dropped any files, Enable MainFrame Toolbar buttons.
        The 'if' statement prevents the call to the function to be unnecessarily 
        recursive, as well as the setting of the variables.
        """
        if invalid:
            return
        if self.switch == 'off':
            self.switch = 'on'
            self.parent.Enable_ToolBtn()
    #----------------------------------------------------------------------
    def deleteAll(self, event):
        """
        Delete and clear all text lines of the TxtCtrl,
        reset the fileList[], disable Toolbar button and menu bar
        Stream/play select imported file - Stream/display imported...
        """
        #self.fileListCtrl.ClearAll()
        self.fileListCtrl.DeleteAllItems()
        del self.fileList[:]
        del duration[:]
        del self.duration[:]
        self.switch = 'off'
        self.parent.Disable_ToolBtn()
        self.parent.importClicked_disable()
        self.selected = False
    #----------------------------------------------------------------------
    def on_select(self, event):
        """
        Selecting a line with mouse or up/down keyboard buttons
        """
        index = self.fileListCtrl.GetFocusedItem()
        item = self.fileListCtrl.GetItemText(index)
        self.parent.importClicked_enable(item)
        self.selected = True
    #----------------------------------------------------------------------
    def on_doubleClick(self, row):
        """
        Double click or keyboard enter button, open media info
        """
        self.onContext(self)
        #self.parent.ImportInfo(self)
    #----------------------------------------------------------------------
    def on_deselect(self, event):
        """
        De-selecting a line with mouse by click in empty space of
        the control list
        """
        self.parent.importClicked_disable()
        self.selected = False
    #----------------------------------------------------------------------
    def on_custom_save(self):
        """
        Choice a specific directory for files save
        """
        dialdir = wx.DirDialog(self, _("Videomass: Choose a directory"))
            
        if dialdir.ShowModal() == wx.ID_OK:
            self.text_path_save.SetValue("")
            self.text_path_save.AppendText(dialdir.GetPath())
            self.file_dest = '%s' % (dialdir.GetPath())
            dialdir.Destroy()
    #----------------------------------------------------------------------
    def same_filedest(self):
        """
        Save destination as files sources 
        """
        if self.ckbx_dir.GetValue() is True:
            self.file_dest = 'same dest'
            self.text_path_save.Disable(), self.btn_save.Disable()
            self.parent.file_save.Enable(False)
            
        elif self.ckbx_dir.GetValue() is False:
            self.file_dest = self.text_path_save.GetValue()
            self.text_path_save.Enable(), self.btn_save.Enable()
            self.parent.file_save.Enable(True)
    #----------------------------------------------------------------------
    
    def statusbar_msg(self, mess, color):
        """
        Set a status bar message of the parent method.
        """
        self.parent.statusbar_msg('%s' % mess, color)
 



# -*- coding: UTF-8 -*-

#########################################################
# Name: presets_mng_panel.py
# Porpose: ffmpeg's presets manager panel
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: Dec 28 2018, Aug.28 2019
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
 
#########################################################

import wx
import os
from videomass3.vdms_IO.presets_manager_properties import parser_xml
from videomass3.vdms_IO.presets_manager_properties import delete_profiles
from videomass3.vdms_IO.presets_manager_properties import supported_formats
from videomass3.vdms_SYS.os_interaction import copy_restore
from videomass3.vdms_SYS.os_interaction import copy_backup
from videomass3.vdms_SYS.os_interaction import copy_on
from videomass3.vdms_IO.filedir_control import inspect
from videomass3.vdms_DIALOGS import presets_addnew
from videomass3.vdms_DIALOGS.epilogue import Formula

array = []# all parameters of the selected profile
# this dictionary content all presets in ~/.videomass:
dict_presets = {
_("Audio Conversions") : ("preset-v1-Audio", "Audio Conversions"), 
_("Extract audio from video"): ("preset-v1-VideoAudio", 
                                "Extract audio from video"),
_("Extract images from video"): ("preset-v1-VideoImages", 
                                "Extract images from video"),
_("Convert to AVI") : ("preset-v1-AVI", "Convert to AVI"),
_("Mobile Phones multimedia") : ("preset-v1-MobilePhones", 
                                 "Mobile Phones multimedia"),
"iPod iTunes" : ("preset-v1-iPod-iTunes", "iPod iTunes"),
_("Convert to VCD (mpg)") : ("preset-v1-VCD", "Convert to VCD (mpg)"),
_("Convert DVD VOB") : ("preset-v1-VOB", "Convert DVD VOB"),
_("Convert to quicktime (mov)") : ("preset-v1-quicktime", 
                                   "Convert to quicktime (mov)"),
_("Convert to DV") : ("preset-v1-DV", "Convert to DV"),
"Google Android" : ("preset-v1-GoogleAndroid", "Google Android"),
"Google webm" : ("preset-v1-GoogleWebm", "Google webm"),
_("DVD Conversions") : ("preset-v1-DVD", "DVD Conversions"),
_("MPEG-4 Conversions") : ("preset-v1-MPEG-4", "MPEG-4 Conversions"),
_("PS3 Compatible") : ("preset-v1-PS3", "PS3 Compatible"),
_("PSP Compatible") : ("preset-v1-PSP", "PSP Compatible"),
_("Websites") : ("preset-v1-websites", "Websites"),
_("User Profiles") : ("preset-v1-Personal", "User Profiles"),
                    }
# set widget colours in some case with html rappresentetion:
azure = '#15a6a6'# rgb form (wx.Colour(217,255,255))
yellow = '#a29500'
red = '#ea312d'
orange = '#f28924'
greenolive = '#8aab3c'

class PresetsPanel(wx.Panel):
    """
    The Presets manager panel is used for the direct execution of profiles 
    with the ffmpeg commands, for editing, restoring and storing them.
    
    """

    def __init__(self, parent, path_srcShare, path_confdir,
                 PWD, threads, cpu_used, ffmpeg_loglev, 
                 ffmpeg_link, OS):
        
        self.src_vdms = os.path.join(path_srcShare, 'vdms')#origine share/vdms
        self.user_vdms = os.path.join(path_confdir, 'vdms')#conf/videomass/vdms
        self.PWD = PWD #current work of videomass
        self.threads = threads
        self.cpu_used = cpu_used if not cpu_used == 'Disabled' else ''
        self.ffmpeg_loglev = ffmpeg_loglev
        self.ffmpeg_link = ffmpeg_link
        self.OS = OS
        self.parent = parent
        self.file_sources = []
        self.file_destin = ''
        self.prstmancmdmod = False # don't show dlg if cmdline is mod manually
        
        wx.Panel.__init__(self, parent, -1) 
        """constructor"""

        self.panel_1 = wx.Panel(self, wx.ID_ANY)

        self.list_ctrl = wx.ListCtrl(self.panel_1, wx.ID_ANY, 
                                        style=wx.LC_REPORT | wx.SUNKEN_BORDER
                                        )
        nb1 = wx.Notebook(self.panel_1, wx.ID_ANY, style=0)
        nb1_p1 = wx.Panel(nb1, wx.ID_ANY)
        lab_prfl = wx.StaticText(nb1_p1, wx.ID_ANY, _("Select a preset from "
                                                    "the drop down:"))
        self.cmbx_prst = wx.ComboBox(nb1_p1,wx.ID_ANY, choices=[
                        (_("Audio Conversions")),
                        (_("Extract audio from video")),
                        (_("Extract images from video")),
                        (_("Convert to AVI")),
                        (_("Mobile Phones multimedia")),
                        ("iPod iTunes"),
                        (_("Convert to VCD (mpg)")),
                        (_("Convert DVD VOB")),
                        (_("Convert to quicktime (mov)")),
                        (_("Convert to DV")),
                        ("Google Android"),
                        ("Google webm"),
                        (_("DVD Conversions")),
                        (_("MPEG-4 Conversions")),
                        (_("PS3 Compatible")),
                        (_("PSP Compatible")),
                        (_("Websites")),
                        (_("User Profiles")),
                        ],
                        style=wx.CB_DROPDOWN | wx.CB_READONLY
                        )
        nb1_p2 = wx.Panel(nb1, wx.ID_ANY)
        self.txt_cmd = wx.TextCtrl(nb1_p2, wx.ID_ANY,"",
                                style=wx.TE_MULTILINE | wx.TE_PROCESS_ENTER)
        
        #----------------------Set Properties----------------------#
        self.cmbx_prst.SetSelection(0)
        
        #self.list_ctrl.SetBackgroundColour(azure)
        self.list_ctrl.SetToolTip(_("List selection profiles"))
        self.txt_cmd.SetMinSize((430, 60))
        self.txt_cmd.SetFont(wx.Font(10, wx.DEFAULT, wx.NORMAL, wx.BOLD, 0, ""))
        self.txt_cmd.SetToolTip(_("Command of the selected profile"))

        #----------------------Build Layout----------------------#
        #siz1 = wx.BoxSizer(wx.VERTICAL)
        siz1 = wx.FlexGridSizer(1, 1, 0, 0)
        grid_siz7 = wx.GridSizer(2, 1, 0, 0)
        grd_s1 = wx.FlexGridSizer(2, 1, 0, 0)
        grd_s2 = wx.FlexGridSizer(3, 1, 0, 0)
        grd_s4 = wx.GridSizer(1, 3, 0, 0)
        grid_siz5 = wx.FlexGridSizer(2, 2, 0, 0)
        grid_siz6 = wx.FlexGridSizer(1, 7, 0, 0)
        grd_s3 = wx.GridSizer(1, 1, 0, 0)
        #siz1.Add(self.DnD, 1, wx.EXPAND|wx.ALL, 10)#########################
        grd_s1.Add(self.list_ctrl, 1, wx.ALL | wx.EXPAND, 15)
        grd_s3.Add(self.txt_cmd, 0, wx.ALL | wx.EXPAND 
                                            | wx.ALIGN_CENTER_HORIZONTAL 
                                            | wx.ALIGN_CENTER_VERTICAL, 15
                                            )
        grid_siz7.Add(lab_prfl, 0, wx.ALIGN_CENTER_HORIZONTAL | wx.ALIGN_CENTER_VERTICAL, 0)
        grid_siz7.Add(self.cmbx_prst, 0, wx.ALIGN_CENTER_HORIZONTAL, 0)
        nb1_p1.SetSizer(grid_siz7)
        nb1_p2.SetSizer(grd_s3)
        nb1.AddPage(nb1_p1, (_("Selecting presets")))
        nb1.AddPage(nb1_p2, (_("Command line FFmpeg")))
        grd_s2.Add(nb1, 1, wx.EXPAND, 0)
        grd_s2.Add(grd_s4, 1, wx.EXPAND, 0)
        grd_s1.Add(grd_s2, 1, wx.ALL | wx.EXPAND, 15)
        self.panel_1.SetSizer(grd_s1)
        siz1.Add(self.panel_1, 1, wx.EXPAND, 0)
        self.SetSizer(siz1)
        siz1.AddGrowableRow(0)
        siz1.AddGrowableCol(0)
        grd_s1.AddGrowableRow(0)
        grd_s1.AddGrowableCol(0)
        grd_s2.AddGrowableRow(0)
        grd_s2.AddGrowableCol(0)
        self.Layout()
        
        #----------------------Binder (EVT)----------------------#
        self.Bind(wx.EVT_COMBOBOX, self.on_choice_profiles, self.cmbx_prst)
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_select, self.list_ctrl)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.parent.Run_Coding, 
                                              self.list_ctrl
                                              )
        self.Bind(wx.EVT_TEXT, self.enter_command, self.txt_cmd)
        
        #----------------------Create preset list----------------------#
        if array != []: # appena avvio, cancella i dati in memoria se esistono
            del array[0:5]

        self.set_listctrl() # passo l'istanza al parsing e creo la lista
    
    #-----------------------------------------------------------------------#
    def disableParent(self):
        """
        disabling the main fraim also automatically disables this panel
        """
        self.parent.Disable()
    #-----------------------------------------------------------------------#
    def enableParent(self):
        """
        Enabling the main fraim also automatically enable this panel
        """
        self.parent.Enable()
    #-----------------------------------------------------------------------#
    def reset_list(self):
        """
        Clear all memory presets and pass to set_listctrl() for 
        re-charging new. This is used when add/edit or delete profiles. 
        The list is reloaded automatically after closing the dialogs for 
        view update .
        """
        self.list_ctrl.ClearAll()
        self.txt_cmd.SetValue("")
        self.set_listctrl()
        if array != []:
            del array[0:5]
            
    def set_listctrl(self):
        """
        Order list of preset (Name/Descript.) from reading xml (vdms)
        """
        try:
            av_presets = dict_presets[self.cmbx_prst.GetValue()][0]
            dati = parser_xml(av_presets, self.user_vdms) # xml parsing
            
            self.list_ctrl.InsertColumn(0, _('Profile Name'), width=230)
            self.list_ctrl.InsertColumn(1, _('Description'), width=350)
            self.list_ctrl.InsertColumn(2, _('Output Extension Type'), width=150)
            self.list_ctrl.InsertColumn(3, _('Supported formats'), width=150)
            
            index = 0
            for name in sorted(dati.keys()):
                rows = self.list_ctrl.InsertItem(index, name)
                self.list_ctrl.SetItem(rows, 0, name)
                param = dati[name]
                self.list_ctrl.SetItem(rows, 1, param["type"])
                self.list_ctrl.SetItem(rows, 2, param["estensione"])
                self.list_ctrl.SetItem(rows, 3, param["filesupport"])
        except:
            UnboundLocalError
            wx.MessageBox(_("For some circumstance, the selected preset is "
                          "corrupted. To restore the original copy at least go "
                          "to the File menu and choose 'restore the preset "
                          "source in use'. Note, before you make this choice, "
                          "saved your personal settings."), "ERROR !", 
                          wx.ICON_ERROR, self)
            return
    #----------------------Event handler (callback)----------------------#
    #------------------------------------------------------------------#
    def on_choice_profiles(self, event): # combobox
        """
        Whenever change preset type in the combobox list, clear all
        in the list_ctrl and re-charge with new file xml, clear the 
        text command and delete elements of array[] if in list.
        """
        self.reset_list()
        
    #------------------------------------------------------------------#
    def on_select(self, event): # list_ctrl
        """
        When select a profile into the list_ctrl, set the parameters
        request.
        Is the main point where be filled the list_ctrl for view.
        The presets are the files with extension vdms. The profiles
        are content it on presets and are selected in list_ctrl
        """
        combvalue = dict_presets[self.cmbx_prst.GetValue()][0] # name xml
        dati = parser_xml(combvalue, self.user_vdms) # All data go in dict
        if array != []:
            del array[0:5] # delete all: lista [0],[1],[2],[3],[4]
            
        slct = event.GetText() # event.GetText is a Name Profile
        self.txt_cmd.SetValue("")
        # param, pass a name profile and search all own elements in list.
        param = dati[event.GetText()] 
        # lista extract and construct command from param and description (type)
        # slct is the profile name
        lista =  [slct, param["type"],param["parametri"], 
                  param["filesupport"], param["estensione"]
                  ]

        array.append(lista[0])# lista[0] is the profile name 
        array.append(lista[1])# lista[1] description
        array.append(lista[2])# lista[2] is the final command
        array.append(lista[3])# lista[3] the file support
        array.append(lista[4])# lista[4] output format name
        
        self.txt_cmd.AppendText(lista[2]) # this is cmd show in text ctrl
        
        self.parent.statusbar_msg(_('Selected profile name:  %s') % (array[0]),
                                                                        None)

    #------------------------------------------------------------------#
    def enter_command(self, event): # text command view
        """
        If a profile is selected, append text into the text_ctrl. Also
        modified the text received signal event
        """
        self.txt_cmd.GetValue()
    #------------------------------------------------------------------#
    def exportStreams(self, exported):
        """
        Set the parent.post_process attribute for communicate it the
        file disponibilities for play or metadata functionalities.
        """
        if not exported:
            return
        else:
            self.parent.post_process = exported
            self.parent.postExported_enable()

    #------------------------------------------------------------------#
    def Saveme(self):
        """
        save a single file copy of preset. The saved presets must be have
        same name where is saved to restore it correctly
        """
        combvalue = dict_presets[self.cmbx_prst.GetValue()][0]
        filedir = '%s/%s.vdms' % (self.user_vdms, combvalue)
        filename = combvalue
        
        dialsave = wx.DirDialog(self, _("Select a directory to save it"))
        if dialsave.ShowModal() == wx.ID_OK:
            dirname = dialsave.GetPath()
            copy_backup(filedir, '%s/%s.vdms' % (dirname, filename))
            dialsave.Destroy()
            wx.MessageBox(_("Successfully saved"), "Info", wx.OK, self)
    #------------------------------------------------------------------#
    def Restore(self):
        """
        Replace preset by another with saved custom profiles.
        """
        wildcard = "Source (*.vdms)|*.vdms| All files (*.*)|*.*"
        
        dialfile = wx.FileDialog(self,
                                 _("Videomass: Choose a videomass preset "
                                   "to restore "), '', "", wildcard, 
                                   wx.FD_OPEN | 
                                   wx.FD_FILE_MUST_EXIST
                                     )
        if dialfile.ShowModal() == wx.ID_OK:
            dirname = dialfile.GetPath()
            tail = os.path.basename(dirname)
            dialfile.Destroy()
            
            for k,v in dict_presets.items():
                if tail in "%s.vdms" % v[0]:
                    vdms = True
                    name = v[1]
                    break
                else:
                    vdms = False
            
            if not vdms:
                wx.MessageBox(_("This file does not match those used "
                                "by Videomass:\n\n'{0}'").format(dirname), 
                                "Videomass",  wx.ICON_INFORMATION, self)
                return
            
            if wx.MessageBox(_("The following preset:\n\n"
                               "'{0}'   ({1})\n\n"
                               "will be imported and will overwrite "
                               "the one in use.\n"
                               "Proceed ?").format(tail, name), 
                             _('Videomass: Please confirm'), 
                                                wx.ICON_QUESTION | 
                                                wx.YES_NO, 
                                                self) == wx.NO:
                return
            
            copy_restore('%s' % (dirname), 
                         '%s/%s' % (self.user_vdms, tail))
            
            self.reset_list() # re-charging functions
    #------------------------------------------------------------------#
    def Default(self):
        """
        Replace the selected preset at default values. 
        This replace new personal changes make at any profiles of the 
        selected preset.
        """ 
        #copy_restore('%s/share/av_presets.xml' % (self.PWD), '%s' % (self.dirconf))
        if wx.MessageBox(_("The selected preset will be overwritten to "
                           "default values!\nproceed?"), 
                           _("Videomass: Please confirm"), 
                            wx.ICON_QUESTION | 
                            wx.YES_NO, self) == wx.NO:
            return
        
        filename = dict_presets[self.cmbx_prst.GetValue()][0]
        copy_restore('%s/%s.vdms' % (self.src_vdms, 
                                     filename), 
                                     '%s/%s.vdms' % (self.user_vdms,
                                                     filename))
        self.reset_list() # re-charging functions
    #------------------------------------------------------------------#
    def Default_all(self):
        """
        restore all preset files in the path presets of the program
        """
        if wx.MessageBox(_("WARNING: you are restoring all the "
                           "default presets!\nProceed?"), 
                           _("Videomass: Please confirm"), 
                            wx.ICON_QUESTION | 
                            wx.YES_NO, self) == wx.NO:
            return

        copy_on('vdms', self.src_vdms, self.user_vdms)
        
        self.reset_list() # re-charging functions
    #------------------------------------------------------------------#
    def Refresh(self):
        """ 
        Pass to reset_list function for re-charging list
        """
        self.reset_list()
    
    #------------------------------------------------------------------#
    def Addprof(self):
        """
        Store new profiles in the same presets selected in the
        combobox. The list is reloaded automatically after
        pressed ok button in the dialog for update view.
        """
        filename = dict_presets[self.cmbx_prst.GetValue()][0]
        name_preset = dict_presets[self.cmbx_prst.GetValue()][1]
        full_pathname = '%s/%s.vdms' % (self.user_vdms, filename)

        prstdialog = presets_addnew.MemPresets(self, 
                                               'newprofile', 
                                               full_pathname, 
                                               filename, 
                                               None, _('Create a new '
                                'profile on the selected  preset "%s"') % (
                                                name_preset)
                                                )
        ret = prstdialog.ShowModal()
        if ret == wx.ID_OK:
            self.reset_list() # re-charging list_ctrl with newer
    #------------------------------------------------------------------#
    def Editprof(self, event):
        """
        A choice in the list (profile) can be edit in all own part.
        The list is reloaded automatically after pressed save button 
        in the dialog.
        """
        if array == []:
            self.parent.statusbar_msg(_("First select a profile in the list"), 
                                        yellow)
            return
        else:
            filename = dict_presets[self.cmbx_prst.GetValue()][0]
            name_preset = dict_presets[self.cmbx_prst.GetValue()][1]
            full_pathname = '%s/%s.vdms' % (self.user_vdms, filename)
            
            prstdialog = presets_addnew.MemPresets(self, 
                                                   'edit', 
                                                   full_pathname, 
                                                   filename, 
                                                   array, 
                                            _('Edit profile on "%s" preset: ') 
                                                   % (name_preset)
                                                   )
            ret = prstdialog.ShowModal()
            if ret == wx.ID_OK:
                self.reset_list() # re-charging list_ctrl with newer
    #------------------------------------------------------------------#
    def Delprof(self):
        """
        Delete a choice in list_ctrl. This delete only single
        profile of the preset used
        """
        if array == []:
            self.parent.statusbar_msg(_("First select a profile in the list"), 
                                         yellow)
        else:
            filename = dict_presets[self.cmbx_prst.GetValue()][0]
            if wx.MessageBox(_("Are you sure you want to delete the "
                             "selected profile?"), 
                             _("Videomass: Please confirm"), 
                             wx.ICON_QUESTION | 
                             wx.YES_NO, self) == wx.NO:
                return
        
            filename = dict_presets[self.cmbx_prst.GetValue()][0]
            # call module-function and pass list as argument
            delete_profiles(array, filename, self.user_vdms)
            self.reset_list()
    #------------------------------------------------------------------#
    def on_ok(self):
        """
        The method composes the command strings to be sent to the 
        ffmpeg's process.
        It also involves the files existence verification procedures,
        overwriting control and log writing.
        
        """
        self.time_seq = self.parent.time_seq
        # make a different id need to avoid attribute overwrite:
        file_sources = self.parent.file_sources[:]
        # make a different id need to avoid attribute overwrite:
        dir_destin = self.file_destin
        # comcheck, cut string in spaces and become list:
        comcheck = self.txt_cmd.GetValue().split()#ffmpeg command
        # used for file name log 
        self.logname = 'Videomass_PresetsManager.log'

        ######## ------------ VALIDAZIONI: --------------
        if array == []:
            self.parent.statusbar_msg(_("First select a profile in the list"),  
                                        yellow)
            return
        
        if array[2].strip() != self.txt_cmd.GetValue().strip():
            if not self.prstmancmdmod:
            
                msg = _("The selected profile command has been "
                        "changed manually.\n"
                        "Do you want to apply it "
                        "during the conversion process?")
                dlg = wx.RichMessageDialog(self, msg, 
                                           _("Videomass: Please confirm"), 
                                            wx.ICON_QUESTION | 
                                            wx.YES_NO,)

                dlg.ShowCheckBox(_("Don't show this dialog again"))

                if dlg.ShowModal() == wx.ID_NO:
                    if dlg.IsCheckBoxChecked():
                        # make sure we won't show it again the next time
                        self.prstmancmdmod = True
                    return
                else:
                    if dlg.IsCheckBoxChecked():
                        # make sure we won't show it again the next time
                        self.prstmancmdmod = True
            
        array3, array4 = array[3], array[4]
        file_sources = supported_formats(array3, file_sources)
        checking = inspect(file_sources, dir_destin, array4)
        if not checking[0]:# l'utente non vuole continuare o files assenti
            return
        # typeproc: batch or single process
        # filename: nome file senza ext.
        # base_name: nome file con ext.
        # countmax: count processing cicles for batch mode
        (typeproc, file_sources, 
         dir_destin, filename, 
         base_name, countmax) = checking

        ######## ------------FINE VALIDAZIONI: --------------
        if array[4] in ['jpg','png','bmp']:
            #self.saveimages(dir_destin)
            self.saveimages(dir_destin, file_sources)
            
        elif 'DOUBLE_PASS' in comcheck:
            
            split = self.txt_cmd.GetValue().split('DOUBLE_PASS')
            passOne = split[0].strip()
            passTwo = split[1].strip()
            
            command1 = ("-loglevel %s %s %s %s -f rawvideo" % (
                                                    self.ffmpeg_loglev, 
                                                    passOne, 
                                                    self.threads, 
                                                    self.cpu_used,
                                                                )
                        )
            command2 = ("-loglevel %s %s %s %s" % (self.ffmpeg_loglev, 
                                                    passTwo, 
                                                    self.threads, 
                                                    self.cpu_used,
                                                    )
                        )
            pass1 = " ".join(command1.split())# mi formatta la stringa
            pass2 = " ".join(command2.split())# mi formatta la stringa
            
            valupdate = self.update_dict(countmax)
            ending = Formula(self, valupdate[0], valupdate[1], _('Starts'))
            if ending.ShowModal() == wx.ID_OK:

                self.parent.switch_Process('doublepass',
                                            file_sources, 
                                            array[4], 
                                            dir_destin, 
                                            None, 
                                            [pass1, pass2], 
                                            self.ffmpeg_link,
                                            '', 
                                            self.logname, 
                                            countmax, 
                                            )
                #used for play preview and mediainfo:
                f = os.path.basename(file_sources[0]).rsplit('.', 1)[0]
                self.exportStreams('%s/%s.%s' % (dir_destin[0], f, 
                                                    array[4]))

        else:
            command = ("-loglevel %s %s %s %s -y" % (self.ffmpeg_loglev, 
                                                    self.txt_cmd.GetValue(), 
                                                    self.threads, 
                                                    self.cpu_used,)
                                                    )
            valupdate = self.update_dict(countmax)
            ending = Formula(self, valupdate[0], valupdate[1], _('Starts'))
            if ending.ShowModal() == wx.ID_OK:
                self.parent.switch_Process('normal',
                                            file_sources, 
                                            array[4], 
                                            dir_destin, 
                                            command, 
                                            None, 
                                            self.ffmpeg_link,
                                            '', 
                                            self.logname, 
                                            countmax, 
                                            )
                f = os.path.basename(file_sources[0]).rsplit('.', 1)[0]
                self.exportStreams('%s/%s.%s' % (dir_destin[0], f, 
                                                    array[4]))
    #--------------------------------------------------------------------#
    def saveimages(self, dest, file_sources):
        """
        Save as files image the selected video input. The saved 
        images are named as file name + a progressive number + .jpg 
        and placed in a folder with the same file name + a progressive 
        number in the chosen output path.
        
        """
        if not self.parent.import_clicked:
            wx.MessageBox(_('To export images, select one of the files '
                            'in the "Add files" panel'), 'Videomass', 
                            wx.ICON_INFORMATION, self)
            return
        
        valupdate = self.update_dict('1')
        ending = Formula(self, valupdate[0], valupdate[1], _('Starts'))
        
        if ending.ShowModal() == wx.ID_OK:
            clicked = self.parent.import_clicked
            fname = os.path.basename(clicked.rsplit('.', 1)[0])
            dir_destin = dest[file_sources.index(clicked)]# specified dest
            
            try: 
                outputdir = "%s/%s-IMAGES_1" % (dir_destin, fname)
                os.mkdir(outputdir)
                
            except FileExistsError:
                lista = []
                for dir_ in os.listdir(dir_destin):
                    if "%s-IMAGES_" % fname in dir_:
                        lista.append(int(dir_.split('IMAGES_')[1]))
                        
                prog = max(lista) +1
                outputdir = "%s/%s-IMAGES_%d" % (dir_destin, fname, prog)
                os.mkdir(outputdir)

            fileout = "{0}-%d.{1}".format(fname,array[4])
            cmd = ('%s %s -i "%s" -loglevel %s %s %s %s -y "%s/%s"' % (
                                                    self.ffmpeg_link, 
                                                    self.parent.time_seq,
                                                    clicked, 
                                                    self.ffmpeg_loglev,
                                                    self.txt_cmd.GetValue(),
                                                    self.threads, 
                                                    self.cpu_used,
                                                    outputdir, 
                                                    fileout)
                                                    )
            command = " ".join(cmd.split())# compact string
            self.parent.switch_Process('saveimages',
                                        clicked, 
                                        None, 
                                        None, 
                                        command, 
                                        None, 
                                        None, 
                                        None, 
                                        self.logname, 
                                        '1', 
                                        )
    #------------------------------------------------------------------#
    #------------------------------------------------------------------#
    
    def update_dict(self, countmax):
        """
        This method is required for update info before send at epilogue
        """
        numfile = "%s file in pending" % str(countmax)
                    
        formula = (_(u"SUMMARY:\n\nFile to Queue\
                      \nProfile Used:\nOut Format:\
                      \nTime selection:"
                      ))
        dictions = ("\n\n%s\n%s\n%s\n%s" % (numfile, 
                                            array[0], 
                                            array[4],
                                            self.time_seq)
                    )

        return formula, dictions




# -*- coding: UTF-8 -*-

#########################################################
# Name: audio_conv
# Porpose: Intarface for ffmpeg audio conversions and normalization
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: Aug.02.2019, Sept.01.2019
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

import wx
import os
import wx.lib.agw.floatspin as FS
import wx.lib.agw.gradientbutton as GB
from videomass3.vdms_IO.IO_tools import volumeDetectProcess
from videomass3.vdms_IO.IO_tools import FFProbe
from videomass3.vdms_IO.filedir_control import inspect
from videomass3.vdms_DIALOGS.epilogue import Formula
from videomass3.vdms_DIALOGS import  audiodialogs
from videomass3.vdms_DIALOGS import presets_addnew
from videomass3.vdms_DIALOGS import shownormlist

# set widget colours in some case with html rappresentetion:
azure = '#15a6a6' # rgb form (wx.Colour(217,255,255))
yellow = '#a29500'
red = '#ea312d'
orange = '#f28924'
greenolive = '#6aaf23'
ciano = '#61ccc7' # rgb 97, 204, 199

cmd_opt = {"AudioContainer":"MP3 [.mp3]", 
           "AudioCodec":"-c:a libmp3lame", 
           "AudioChannel":("",""), 
           "AudioRate":("",""), 
           "AudioDepth":("",""),
           "AudioBitrate":("",""), 
           "Normalize":"", 
           "ExportExt":"mp3", 
           "CodecCopied":[],
           }
acodecs = {"WAV [.wav]":"-c:a pcm_s16le",
           "AIFF [.aiff]":"-c:a pcm_s16le",
           "FLAC [.flac]":"-c:a flac",
           "OGG [.ogg]":"-c:a libvorbis",
           "MP3 [.mp3]":"-c:a libmp3lame",
           "AAC [.m4a]":"-c:a aac",
           "ALAC [.m4a]":"-c:a alac",
           "AC3 [.ac3]":"-c:a ac3"
           }

class Audio_Conv(wx.Panel):
    """
    Interface panel for audio conversions and volume normalizations,
    with preset storing feature (TODO)
    """
    def __init__(self, parent, ffmpeg_link, threads, 
                 cpu_used, ffmpeg_loglev, ffprobe_link, OS,
                 iconanalyzes, iconsettings):
        # passed attributes
        self.parent = parent
        self.ffmpeg_link = ffmpeg_link
        self.threads = threads
        self.cpu_used = cpu_used if not cpu_used == 'Disabled' else ''
        self.ffmpeg_loglevel = ffmpeg_loglev
        self.ffprobe_link = ffprobe_link
        self.OS = OS
        #self.DIRconf = DIRconf
        # others attributes:
        self.file_sources = []
        self.file_destin = ''
        self.normdetails = []

        wx.Panel.__init__(self, parent, -1)
        """ constructor"""
        # Widgets definitions:
        self.cmbx_a = wx.ComboBox(self, wx.ID_ANY,
        choices=[("WAV [.wav]"), 
                 ("AIFF [.aiff]"), 
                 ("FLAC [.flac]"), 
                 ("OGG [.ogg]"), 
                 ("MP3 [.mp3]"), 
                 ("AAC [.m4a]"), 
                 ("ALAC [.m4a]"), 
                 ("AC3 [.ac3]"),
                 (_("Save audio from movie"))], style=wx.CB_DROPDOWN | 
                                                       wx.CB_READONLY)
        self.cmbx_a.SetSelection(4)
        setbmp = wx.Bitmap(iconsettings, wx.BITMAP_TYPE_ANY)
        self.btn_param = GB.GradientButton(self,
                                           size=(-1,25),
                                           bitmap=setbmp,
                                           label=_("Audio Options"))
        self.btn_param.SetBaseColours(startcolour=wx.Colour(158,201,232),
                                    foregroundcolour=wx.Colour(28,28,28))
        self.btn_param.SetBottomEndColour(wx.Colour(205, 235, 222))
        self.btn_param.SetBottomStartColour(wx.Colour(205, 235, 222))
        self.btn_param.SetTopStartColour(wx.Colour(205, 235, 222))
        self.btn_param.SetTopEndColour(wx.Colour(205, 235, 222))
        
        self.txt_options = wx.TextCtrl(self, wx.ID_ANY, size=(265,-1),
                                          style=wx.TE_READONLY)
        self.ckb_onlynorm = wx.CheckBox(self, wx.ID_ANY, (
                                               _("Only Normalization")))
        self.ckb_norm = wx.CheckBox(self, wx.ID_ANY, (
                                               _("Audio Normalization")))
        analyzebmp = wx.Bitmap(iconanalyzes, wx.BITMAP_TYPE_ANY)
        self.btn_analyzes = GB.GradientButton(self,
                                           size=(-1,25),
                                           bitmap=analyzebmp,
                                           label=_("Volumedected"))
        self.btn_analyzes.SetBaseColours(startcolour=wx.Colour(158,201,232),
                                    foregroundcolour=wx.Colour(165,165, 165))
        self.btn_analyzes.SetBottomEndColour(wx.Colour(205, 235, 222))
        self.btn_analyzes.SetBottomStartColour(wx.Colour(205, 235, 222))
        self.btn_analyzes.SetTopStartColour(wx.Colour(205, 235, 222))
        self.btn_analyzes.SetTopEndColour(wx.Colour(205, 235, 222))
        
        self.btn_details = GB.GradientButton(self,
                                            #size=(-1,25),
                                            #bitmap=analyzebmp,
                                            label=_("Details list"))
        self.btn_details.SetBaseColours(startcolour=wx.Colour(158,201,232),
                                    foregroundcolour=wx.Colour(165,165, 165))
        self.btn_details.SetBottomEndColour(wx.Colour(205, 235, 222))
        self.btn_details.SetBottomStartColour(wx.Colour(205, 235, 222))
        self.btn_details.SetTopStartColour(wx.Colour(205, 235, 222))
        self.btn_details.SetTopEndColour(wx.Colour(205, 235, 222))
        
        self.lab_amplitude = wx.StaticText(self, wx.ID_ANY, (
                                    _("Max peak level threshold   ")))
        self.spin_amplitude = FS.FloatSpin(self, wx.ID_ANY, min_val=-99.0, 
                                    max_val=0.0, increment=1.0, value=-1.0, 
                                    agwStyle=FS.FS_LEFT,size=(-1,-1))
        self.spin_amplitude.SetFormat("%f")
        self.spin_amplitude.SetDigits(1)
        #----------------------Set Properties----------------------#
        sizer_base = wx.BoxSizer(wx.VERTICAL)
        sizer_global = wx.StaticBoxSizer(wx.StaticBox(self, wx.ID_ANY, (
                                "")), wx.VERTICAL)
        sizer_2 = wx.BoxSizer(wx.HORIZONTAL)
        grid_sizer_3 = wx.FlexGridSizer(5, 2, 0, 0)
        grid_sizer_1 = wx.GridSizer(2, 1, 0, 0)
        grid_sizer_2 = wx.FlexGridSizer(1, 2, 0, 0)
        grid_sizer_4 = wx.FlexGridSizer(2, 4, 0, 0)
        #grid_sizer_5 = wx.FlexGridSizer(1, 3, 0, 0)
        sizer_3 = wx.StaticBoxSizer(wx.StaticBox(self, wx.ID_ANY, (
                                    _("Audio Container Selection"))), 
                                                       wx.VERTICAL
                                                       )
        sizer_3.Add(self.cmbx_a, 0, wx.ALIGN_CENTER | wx.ALL, 20)
        grid_sizer_1.Add(sizer_3, 1, wx.ALL | wx.EXPAND, 5)
        grid_sizer_2.Add(self.btn_param, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5)
        grid_sizer_2.Add(self.txt_options, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5)
        #grid_sizer_4.Add(self.lab_bitdepth, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5)
        #grid_sizer_4.Add(self.txt_bitdepth, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5)
        #grid_sizer_4.Add(self.lab_bitrate, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5)
        #grid_sizer_4.Add(self.txt_bitrate, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5)
        #grid_sizer_4.Add(self.lab_channel, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5)
        #grid_sizer_4.Add(self.txt_channel, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5)
        #grid_sizer_2.Add(grid_sizer_4, 0, wx.TOP, 5)
        grid_sizer_1.Add(grid_sizer_2, 1, 0, 0)
        sizer_2.Add(grid_sizer_1, 1, wx.ALL | wx.EXPAND, 20)
        grid_sizer_3.Add(self.ckb_norm, 0, wx.TOP, 5)
        grid_sizer_3.Add(self.ckb_onlynorm, 0, wx.TOP, 5)
        grid_sizer_3.Add(self.btn_analyzes, 0, wx.TOP, 10)
        grid_sizer_3.Add((20, 20), 0, wx.EXPAND | wx.TOP, 5)
        grid_sizer_3.Add(self.btn_details, 0, wx.TOP, 10)
        grid_sizer_3.Add((20, 20), 0, wx.EXPAND | wx.TOP, 5)
        grid_sizer_3.Add(self.lab_amplitude, 0, wx.TOP, 10)
        grid_sizer_3.Add(self.spin_amplitude, 0, wx.TOP, 5)
        #--------------------------------------------------------------
        grid_sizer_3.Add((0, 55), 0, wx.EXPAND | wx.TOP, 5)
        grid_sizer_3.Add((0, 55), 0, wx.EXPAND | wx.TOP, 5)
        #sizer_4 = wx.StaticBoxSizer(wx.StaticBox(self, wx.ID_ANY, (
                                    #"Export Preferences")), wx.VERTICAL)
        #sizer_4.Add(grid_sizer_5, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5)
        #grid_sizer_5.Add(self.ckb_test, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL,5)
        #grid_sizer_5.Add(self.label_sec, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL,5)
        #grid_sizer_5.Add(self.spin_ctrl_test, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL,5)
        #grid_sizer_3.Add(sizer_4, 0, wx.TOP|wx.BOTTOM, 20)
        #--------------------------------------------------------------
        sizer_2.Add(grid_sizer_3, 1, wx.ALL | wx.EXPAND, 20)
        sizer_global.Add(sizer_2, 1, wx.ALL | wx.EXPAND, 5)
        sizer_base.Add(sizer_global, 1, wx.ALL | wx.EXPAND, 5)
        self.SetSizer(sizer_base)
        # Set tooltip:
        self.btn_param.SetToolTip(_("Enable advanced settings as audio "
                                    "bit-rate, audio channel and audio rate "
                                    "of the selected audio codec.")
                                              )
        self.btn_analyzes.SetToolTip(_("Calculates the maximum and average "
                                       "peak level of audio streams expressed "
                                       "in dB values")
                                              )
        self.spin_amplitude.SetToolTip(_("Threshold for the maximum peak "
                                    "level in dB values. The default setting "
                                    "is -1.0 dB and is good for most of the "
                                    "processes."))
        
        #----------------------Binding (EVT)----------------------#
        self.cmbx_a.Bind(wx.EVT_COMBOBOX, self.audioFormats)
        self.Bind(wx.EVT_BUTTON, self.on_Param, self.btn_param)
        self.Bind(wx.EVT_CHECKBOX, self.on_Enable_norm, self.ckb_norm)
        self.Bind(wx.EVT_CHECKBOX, self.on_OnlyNorm, self.ckb_onlynorm)
        self.Bind(wx.EVT_BUTTON, self.on_Analyzes, self.btn_analyzes)
        self.Bind(wx.EVT_BUTTON, self.on_Show_normlist, self.btn_details)
        #self.Bind(wx.EVT_SPINCTRL, self.enter_normalize, self.spin_amplitude)
        
        # set default :
        self.normalization_disabled()

    # used Methods:
    def normalization_disabled(self):
        """
        Disable all widget of normalization feature and Set default values. 
        This is the start default setting and when there are changes in the 
        dragNdrop panel. Also, if enable 'Perform only normalization' enable 
        conversion widgets.
        """
        self.ckb_norm.SetValue(False), self.btn_analyzes.Disable(), 
        self.btn_details.Disable()
        self.btn_details.SetForegroundColour(wx.Colour(165,165, 165))
        self.lab_amplitude.Disable()
        self.spin_amplitude.Disable(), self.spin_amplitude.SetValue(-1.0)
        cmd_opt["Normalize"] = ""
        del self.normdetails[:]
        if self.ckb_onlynorm.IsChecked():
            self.ckb_onlynorm.SetValue(False)
            self.cmbx_a.Enable(), self.btn_param.Enable(),
            self.ckb_norm.Enable(), self.txt_options.Enable(),

    #----------------------Event handler (callback)----------------------#
    #------------------------------------------------------------------#
    
    def audioFormats(self, evt):
        """
        Get selected container from combobox
        """
        if self.cmbx_a.GetValue() == _("Save audio from movie"):
            if self.ckb_norm.IsChecked():
                self.normalization_disabled()
            self.ckb_norm.Disable(), self.ckb_onlynorm.Disable()
            self.txt_options.Disable(), self.btn_param.Disable()
            self.btn_param.SetForegroundColour(wx.Colour(165,165, 165))
            
        else:
            if not self.ckb_norm.IsChecked():# è l'unico caso questo
                self.ckb_norm.Enable(), self.ckb_onlynorm.Enable(), 
                self.txt_options.Enable(), self.btn_param.Enable()
                self.btn_param.SetForegroundColour(wx.Colour(28,28,28))
                
            cmd_opt["AudioContainer"] = self.cmbx_a.GetValue()
            cmd_opt["AudioCodec"] = acodecs[self.cmbx_a.GetValue()]
            ext = self.cmbx_a.GetValue().split()[1].strip('[.]')
            cmd_opt["ExportExt"] = ext
        
        self.txt_options.SetValue("")
        cmd_opt["AudioChannel"] = ["",""]
        cmd_opt["AudioRate"] = ["",""]
        cmd_opt["AudioBitrate"] = ["",""]
        cmd_opt["AudioDepth"] = ["",""]
        self.btn_param.SetBottomEndColour(wx.Colour(205, 235, 222))
    #------------------------------------------------------------------#
    def on_Param(self, evt):
        """
        Get the container type and set value dictionary codec in the
        AudioCodec key used for ffmpeg command..
        Send identifier data of the container to audio_parameters method
        """
        if not cmd_opt["AudioContainer"]:
            cmd_opt["AudioContainer"] = self.cmbx_a.GetValue()
        #--------------------------------------------#
        if cmd_opt["AudioContainer"] == "WAV [.wav]":
            self.audio_parameters("wav", "Audio WAV Parameters - Videomass")
            
        elif cmd_opt["AudioContainer"] == "AIFF [.aiff]":
            self.audio_parameters("aiff", "Audio AIFF Parameters - Videomass")

        elif cmd_opt["AudioContainer"] == "FLAC [.flac]":
            self.audio_parameters("flac", "Audio FLAC Parameters - Videomass")
            
        elif cmd_opt["AudioContainer"] == "OGG [.ogg]":
            self.audio_parameters("ogg", "Audio OGG Parameters - Videomass")
            
        elif cmd_opt["AudioContainer"] == "MP3 [.mp3]":
            self.audio_parameters("mp3", "Audio MP3 Parameters - Videomass")
        
        elif cmd_opt["AudioContainer"] == "AAC [.m4a]":
            self.audio_parameters("aac", "Audio AAC Parameters - Videomass")
        
        elif cmd_opt["AudioContainer"] == "ALAC [.m4a]":
            self.audio_parameters("alac", "Audio ALAC Parameters - Videomass")
            
        elif cmd_opt["AudioContainer"] == "AC3 [.ac3]":
            self.audio_parameters("ac3", "Audio AC3 Parameters - Videomass")
            
        #elif self.cmbx_a.GetValue() == "Save audio from movie":
            #self.audiocopy()
    #-------------------------------------------------------------------#
    def audio_parameters(self, audio_type, title):
        """
        Run a dialogs to choices audio parameters then set dictionary 
        at proper values. Also text is placed in the textctrl's fields.
        NOTE: The data[X] tuple contains the command parameters on the index [1] 
              and the descriptive parameters on the index [0].
              exemple: data[0] contains parameters for channel then
              data[0][1] is ffmpeg option command for audio channels and
              data[0][0] is a simple description for view.
        """
        audiodialog = audiodialogs.AudioSettings(self, 
                                                 audio_type, 
                                                 cmd_opt["AudioRate"],
                                                 cmd_opt["AudioDepth"],
                                                 cmd_opt["AudioBitrate"], 
                                                 cmd_opt["AudioChannel"],
                                                 title,
                                                 )
        retcode = audiodialog.ShowModal()
        
        if retcode == wx.ID_OK:
            data = audiodialog.GetValue()
            cmd_opt["AudioChannel"] = data[0]
            cmd_opt["AudioRate"] = data[1]
            cmd_opt["AudioBitrate"] = data[2]
            if audio_type in ('wav','aiff'):
                if 'Default' in data[3][0]:
                    cmd_opt["AudioCodec"] = "-c:a pcm_s16le"
                else:
                    cmd_opt["AudioCodec"] = data[3][1]
                cmd_opt["AudioDepth"] = ("%s" % (data[3][0]),
                                         "%s" % (data[3][1]))
            else:# entra su tutti gli altri tranne wav aiff
                cmd_opt["AudioDepth"] = data[3]
        else:
            data = None
            audiodialog.Destroy()
            return

        self.txt_options.SetValue("")
        count = 0
        for d in [cmd_opt["AudioRate"],cmd_opt["AudioDepth"],
                 cmd_opt["AudioBitrate"], cmd_opt["AudioChannel"]
                 ]:
            if d[1]:
                count += 1
                self.txt_options.AppendText(" %s | " % d[0])

        if count == 0:
            self.btn_param.SetBottomEndColour(wx.Colour(205, 235, 222))
        else:
            self.btn_param.SetBottomEndColour(wx.Colour(0, 240, 0))
        
        audiodialog.Destroy()
    #------------------------------------------------------------------#
    def audiocopy(self,file_sources):
        """
        Try copying and saving the audio stream on a video by 
        recognizing the codec and then assigning a container/format. 
        If there is more than one audio stream in a video, a choice 
        is made to the user.
        """
        cmd_opt["ExportExt"] = []
        cmd_opt["AudioCodec"] = []
        cmd_opt["CodecCopied"] = []

        for files in file_sources:
            metadata = FFProbe(files, self.ffprobe_link, '-pretty') 
            # first execute a control for errors:
            if metadata.ERROR():
                wx.MessageBox("[FFprobe] Error:  %s" % (metadata.error), 
                              "ERROR - Videomass",
                wx.ICON_ERROR, self)
                return
            # Proceed with the istance method call:
            datastream = metadata.get_audio_codec_name()
            audio_list = datastream[0]

            if audio_list == None:
                wx.MessageBox(_("There are no audio streams:\n%s ") % (files), 
                            'Videomass', wx.ICON_INFORMATION, self)
                return

            elif len(audio_list) > 1:
                title = datastream[1]
                
                dlg = wx.SingleChoiceDialog(self, 
                        _("{0}\n\n"
                          "Contains multiple audio streams. " 
                          "Select which audio stream you want to "
                          "export between these:").format(title), 
                        _("Videomass: Stream choice"),
                        audio_list, wx.CHOICEDLG_STYLE
                                            )
                if dlg.ShowModal() == wx.ID_OK:
                    if dlg.GetStringSelection() in audio_list:
                        cn = ''.join(dlg.GetStringSelection()).split()[4]
                        indx = ''.join(dlg.GetStringSelection()).split()[1]
                        cmd_opt["AudioCodec"].append(
                                            "-map 0:%s -c copy" % indx)
                        cmd_opt["CodecCopied"].append(cn)#used for epilogue
                        if cn in ('aac','alac'): cn = 'm4a'
                        if cn in ('ogg','vorbis'): cn = 'oga'
                        if cn.startswith('pcm_'): cn = 'wav'
                        cmd_opt["ExportExt"].append(cn)
                    else:
                        wx.MessageBox(_(u"Nothing choice:\n%s ") % (files), 
                            'Videomass: Error', wx.ICON_ERROR, self)
                        return
                else: # there must be some choice (default first item list)
                    cn = ''.join(audio_list[0]).split()[4]
                    indx = ''.join(audio_list[0]).split()[1]
                    cmd_opt["AudioCodec"].append("-map 0:%s -c copy" % indx)
                    cmd_opt["CodecCopied"].append(cn)#used for epilogue
                    if cn in ('aac','alac'): cn = 'm4a'
                    if cn in ('ogg','vorbis'): cn = 'oga'
                    if cn.startswith('pcm_'): cn = 'wav'
                    cmd_opt["ExportExt"].append(cn)
                dlg.Destroy()
            else:
                cn = ''.join(audio_list[0]).split()[4]
                indx = ''.join(audio_list[0]).split()[1]
                cmd_opt["AudioCodec"].append("-map 0:%s -c copy" % indx)
                cmd_opt["CodecCopied"].append(cn)#used for epilogue
                if cn in ('aac','alac'): cn = 'm4a'
                if cn in ('ogg','vorbis'): cn = 'oga'
                if cn.startswith('pcm_'): cn = 'wav'
                cmd_opt["ExportExt"].append(cn)

    #------------------------------------------------------------------#
    def on_Enable_norm(self, evt):
        """
        Choice if use or not audio normalization
        """
        msg = (_('Tip: set the maximum peak level threshold or accept default '
                 'dB value (-1.0); then check peak level by pressing the '
                 '"Volumedetect" button'))
        if self.ckb_norm.IsChecked():# if checked
            self.parent.statusbar_msg(msg, azure)
            self.btn_analyzes.SetForegroundColour(wx.Colour(28,28,28))
            self.btn_analyzes.Enable(), self.spin_amplitude.Enable(),
            self.lab_amplitude.Enable(), 

        else:# is not checked
            self.parent.statusbar_msg(_("Disable audio normalization"), None)
            self.btn_analyzes.SetForegroundColour(wx.Colour(165,165, 165))
            self.btn_analyzes.Disable(), self.lab_amplitude.Disable()
            self.spin_amplitude.Disable(), self.spin_amplitude.SetValue(-1.0)
            self.btn_details.SetForegroundColour(wx.Colour(165,165, 165))
            self.btn_details.Disable()
            
        cmd_opt["Normalize"] = ""
        del self.normdetails[:]
    #------------------------------------------------------------------#
    def on_OnlyNorm(self, evt):
        """
        If checked, disable audio conversion and use only normalization
        process to audio file
        """
        if self.ckb_onlynorm.IsChecked():# if checked
            if not self.ckb_norm.IsChecked():
                self.ckb_norm.SetValue(True), self.on_Enable_norm(self)
            self.ckb_norm.Disable()
            self.cmbx_a.Disable(), self.btn_param.Disable(),
            self.txt_options.Disable(),
            self.btn_param.SetForegroundColour(wx.Colour(165,165, 165))
        else:
            self.cmbx_a.Enable(), self.btn_param.Enable(),
            self.btn_param.SetForegroundColour(wx.Colour(28,28, 28))
            self.ckb_norm.Enable(),
            self.txt_options.Enable(),
    #------------------------------------------------------------------#
    def on_Analyzes(self, evt):  # analyzes button
        """
        If check, send to audio_analyzes.
        """
        file_sources = self.parent.file_sources[:]
        self.audio_analyzes(file_sources)
    #------------------------------------------------------------------#
    
    def audio_analyzes(self, file_sources):
        """
        Start control for values volume in db. 
        Then call 'os_processing.proc_volumedetect'.
        <https://superuser.com/questions/323119/how-can-i-normalize-audio-
        using-ffmpeg?utm_medium=organic>
        """
        msg1 = (_("Audio normalization will be applied"))
        msg2 = (_("Audio normalization is required only for some files"))
        msg3 = (_("Audio normalization is not required in relation to "
                  "the set threshold"))
        
        self.parent.statusbar_msg("",None)
        self.time_seq = self.parent.time_seq #from -ss to -t will be analyzed
        normalize = self.spin_amplitude.GetValue()

        data = volumeDetectProcess(self.ffmpeg_link, 
                                   file_sources, 
                                   self.time_seq)
        if data[1]:
            wx.MessageBox(data[1], "Volumedected error!", wx.ICON_ERROR)
            return
        else:
            volume = list()

            for f, v in zip(file_sources, data[0]):
                maxvol = v[0].split(' ')[0]
                meanvol = v[1].split(' ')[0]
                offset = float(maxvol) - float(normalize)
                if float(maxvol) >= float(normalize):
                    volume.append('  ')
                    self.normdetails.append((f, 
                                             maxvol, 
                                             meanvol,
                                             ' ',
                                             _('Not Required')
                                             ))
                else:
                    volume.append("-af volume=%sdB" % (str(offset)[1:]))
                    self.normdetails.append((f, 
                                             maxvol,
                                             meanvol,
                                             str(offset)[1:],
                                             _('Required')
                                             ))
                    
        if [a for a in volume if not '  ' in a] == []:
             self.parent.statusbar_msg(msg3, orange)
        else:
            if len(volume) == 1 or not '  ' in volume:
                 self.parent.statusbar_msg(msg1, greenolive)
            else:
                self.parent.statusbar_msg(msg2, yellow)
                
        cmd_opt["Normalize"] = volume
        self.btn_analyzes.Disable()
        self.btn_analyzes.SetForegroundColour(wx.Colour(165,165, 165))
        self.btn_details.Enable()
        self.btn_details.SetForegroundColour(wx.Colour(28,28,28))
        
    #------------------------------------------------------------------#
    def on_Show_normlist(self, event):
        """
        Show a wx.ListCtrl dialog to list data of peak levels
        """
        title = _('Audio normalization details list')
        audionormlist = shownormlist.NormalizationList(title, 
                                                       self.normdetails, 
                                                       self.OS)
        audionormlist.Show()
    #-----------------------------------------------------------------------#
    def disableParent(self):
        """
        disabling the main fraim also automatically disables this panel.
        Used on batch mode only with ProgressDialog gauge. 
        """
        self.parent.Disable()
    #-----------------------------------------------------------------------#
    def enableParent(self):
        """
        Enabling the main fraim also automatically enable this panel
        """
        self.parent.Enable()
    #------------------------------------------------------------------#
    def exportStreams(self, exported):
        """
        Set the parent.post_process attribute for communicate it the
        file disponibilities for play or metadata functionalities.
        """
        if not exported:
            return

        else:
            self.parent.post_process = exported
            self.parent.postExported_enable()
    #------------------------------------------------------------------#
    def update_allentries(self):
        """
        Last step for set definitively all values before to proceed
        with std_conv or batch_conv methods.
        Update _allentries is callaed by on_ok method.
        """
        self.time_seq = self.parent.time_seq
        if self.cmbx_a.GetValue() == _("Save audio from movie"):
            file_sources = self.parent.file_sources[:]
            self.audiocopy(file_sources)
    #------------------------------------------------------------------#
    def on_ok(self):
        """
        File existence verification procedures, overwriting control and 
        data redirecting .
        """
        # check normalization data offset, if enable.
        if self.ckb_norm.IsChecked():
            if self.btn_analyzes.IsEnabled():
                wx.MessageBox(_("Missing volume dectect!\n"
                              "Press the Volumedected button before proceeding."),
                                "Videomass", wx.ICON_INFORMATION)
                return
        self.update_allentries()# last update of all setting interface
        # make a different id need to avoid attribute overwrite:
        file_sources = self.parent.file_sources[:]
        # make a different id need to avoid attribute overwrite:
        dir_destin = self.file_destin
        # used for file name log 
        logname = 'Videomass_AudioConversion.log'

        ######## ------------ VALIDAZIONI: --------------
        if self.ckb_onlynorm.IsChecked():
            checking = inspect(file_sources, dir_destin, '')
        else:
            checking = inspect(file_sources, dir_destin, 
                            cmd_opt["ExportExt"])
        if not checking[0]:#user non vuole continuare o files assenti
            return
        # typeproc: batch or single process
        # filename: nome file senza ext.
        # base_name: nome file con ext.
        # countmax: count processing cicles for batch mode
        typeproc, file_sources, dir_destin,\
        filename, base_name, countmax = checking


        if self.cmbx_a.GetValue() == _("Save audio from movie"):
            self.grabaudioProc(file_sources, dir_destin, countmax, logname)
        else:
            self.stdProc(file_sources, dir_destin, countmax, logname)

    #------------------------------------------------------------------#
    def stdProc(self, file_sources, dir_destin, countmax, logname):
        """
        Composes the ffmpeg command strings for the batch mode processing.
        """
        if self.ckb_onlynorm.IsChecked():
            title = _('Start audio norm.')
            cmd = ("-loglevel %s -vn %s %s -y" % (self.ffmpeg_loglevel, 
                                                  self.threads,
                                                  self.cpu_used,)
                                                  )
            command = " ".join(cmd.split())# mi formatta la stringa
            valupdate = self.update_dict(countmax)
            ending = Formula(self, valupdate[0], valupdate[1], title)
            
            if ending.ShowModal() == wx.ID_OK:
                self.parent.switch_Process('normal',
                                           file_sources, 
                                           '', 
                                           dir_destin, 
                                           command, 
                                           None, 
                                           self.ffmpeg_link,
                                           cmd_opt["Normalize"], 
                                           logname, 
                                           countmax, 
                                           )
                #used for play preview and mediainfo:
                f = '%s/%s' % (dir_destin[0], os.path.basename(file_sources[0]))
                self.exportStreams(f)#call function more above
        else:
            title = _('Start audio conversion')
            command = ("-loglevel %s -vn %s %s %s %s %s %s %s -y" % (
                                                self.ffmpeg_loglevel,
                                                cmd_opt["AudioCodec"],
                                                cmd_opt["AudioBitrate"][1], 
                                                cmd_opt["AudioDepth"][1], 
                                                cmd_opt["AudioRate"][1], 
                                                cmd_opt["AudioChannel"][1], 
                                                self.threads,
                                                self.cpu_used,)
                                                                    )
            command = " ".join(command.split())# mi formatta la stringa
            valupdate = self.update_dict(countmax)
            ending = Formula(self, valupdate[0], valupdate[1], title)

            if ending.ShowModal() == wx.ID_OK:
                self.parent.switch_Process('normal',
                                           file_sources,
                                           cmd_opt["ExportExt"],
                                           dir_destin,
                                           command,
                                           None,
                                           self.ffmpeg_link,
                                           cmd_opt["Normalize"],
                                           logname, 
                                           countmax,
                                           )
                #used for play preview and mediainfo:
                f = os.path.basename(file_sources[0]).rsplit('.', 1)[0]
                self.exportStreams('%s/%s.%s' % (dir_destin[0], f, 
                                                 cmd_opt["ExportExt"]))
        
    #------------------------------------------------------------------#
    def grabaudioProc(self, file_sources, dir_destin, 
                             countmax, logname):
        """
        Composes the ffmpeg command strings for the batch_process_changes.
        """
        title = _('Start audio export')
        cmdsplit1 = ("-loglevel %s -vn" % (self.ffmpeg_loglevel,))
        cmdsplit2 = ("%s %s -y" % (self.threads, self.cpu_used,))

        valupdate = self.update_dict(countmax)
        ending = Formula(self, valupdate[0], valupdate[1], title)
        
        if ending.ShowModal() == wx.ID_OK:
            self.parent.switch_Process('grabaudio',
                                       self.ffmpeg_link, 
                                       file_sources, #list
                                       dir_destin, #list
                                       cmdsplit1, 
                                       cmd_opt["AudioCodec"],#list
                                       cmdsplit2, 
                                       cmd_opt["ExportExt"],#list
                                       logname, 
                                       countmax, #list
                                       )
            # used for play preview and mediainfo:
            f = os.path.basename(file_sources[0]).rsplit('.', 1)[0]
            self.exportStreams('%s/%s.%s' % (dir_destin[0], f, 
                                             cmd_opt["ExportExt"][0]))

    #------------------------------------------------------------------#
    def update_dict(self, countmax):
        """
        This method is required for update all cmd_opt
        dictionary values before send at epilogue
        """
        numfile = _("%s file in pending") % str(countmax)
        if cmd_opt["Normalize"]:
            normalize = _('Enable')
        else:
            normalize = _('Disable')
            
        if self.ckb_onlynorm.IsChecked():
            formula = (_("SUMMARY:\n\nFile Queue:\
                       \nAudio Normalization:\nTime selection:"))
            dictions = ("\n\n%s\n%s\n%s" % (numfile, 
                                            normalize, 
                                            self.time_seq
                                            )
                        )
        elif self.cmbx_a.GetValue() == _("Save audio from movie"):
            formula = (_("SUMMARY:\n\nFile Queue:\
                      \nAudio Container:\nCodec copied:\nTime selection:"))
            dictions = ("\n\n%s\n%s\n%s\n%s" % (numfile, 
                                                cmd_opt["ExportExt"],
                                                cmd_opt["CodecCopied"], 
                                                self.time_seq)
                                                )
        else:
            formula = (_("SUMMARY:\n\nFile Queue:\
                    \nAudio Container:\nAudio Codec:\nAudio bit-rate:\
                    \nAudio channel:\nAudio sample rate:\nBit per Sample:\
                    \nAudio Normalization:\nTime selection:"))
            dictions = ("\n\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s" % (
                    numfile, cmd_opt["AudioContainer"], 
                    cmd_opt["AudioCodec"], cmd_opt["AudioBitrate"][0], 
                    cmd_opt["AudioChannel"][0], cmd_opt["AudioRate"][0], 
                    cmd_opt["AudioDepth"][0] , normalize, self.time_seq)
                        )
        return formula, dictions
    
#------------------------------------------------------------------#
    def Addprof(self):
        """
        Storing new profile in the 'Preset Manager' panel with the same 
        current setting. All profiles saved in this way will also be stored 
        in the preset 'User Presets'
        
        NOTE: For multiple processes involving audio normalization and those 
              for saving audio streams from movies, only the data from the 
              first file in the list will be considered.
        
        FIXME have any problem with xml escapes in special character
        (like && for ffmpeg double pass), so there is some to get around it 
        (escamotage), but work .
        
        """
        get = wx.GetApp()
        dirconf = os.path.join(get.DIRconf, 'vdms')
        
        if cmd_opt["Normalize"]:
            normalize = cmd_opt["Normalize"][0]# tengo il primo valore lista
        else:
            normalize = ''
        
        if self.ckb_onlynorm.IsChecked():
            command = ("-vn %s" % normalize)
            command = ' '.join(command.split())# sistemo gli spazi
            list = [command, cmd_opt["ExportExt"]]
            
        elif self.cmbx_a.GetValue() == _("Save audio from movie"):
            command = ("-vn %s" % cmd_opt["AudioCodec"][0])
            command = ' '.join(command.split())# sistemo gli spazi
            list = [command, cmd_opt["ExportExt"][0]]
  
        else:
            command = ("-vn %s %s %s %s %s %s" % (normalize, 
                                                  cmd_opt["AudioCodec"], 
                                                  cmd_opt["AudioBitrate"][1], 
                                                  cmd_opt["AudioDepth"][1], 
                                                  cmd_opt["AudioRate"][1], 
                                                  cmd_opt["AudioChannel"][1],
                                                  ))
            command = ' '.join(command.split())# sistemo gli spazi
            list = [command, cmd_opt["ExportExt"]]

        filename = 'preset-v1-Personal'# nome del file preset senza ext
        name_preset = 'User Profiles'
        full_pathname = os.path.join(dirconf, 'preset-v1-Personal.vdms')
        
        prstdlg = presets_addnew.MemPresets(self, 'addprofile', full_pathname, 
                                            filename, list, 
                    _('Videomass: Create a new profile on "%s" preset') % (
                                                                name_preset))
        prstdlg.ShowModal()



# -*- coding: UTF-8 -*-

#########################################################
# Name: infoprog.py
# Porpose: about videomass 
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigoto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2019 Gianluca Pernigoto <jeanlucperni@gmail.com>
# license: GPL3
# Rev (01) December 28 2018
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################

import wx
import wx.adv
from videomass3.vdms_SYS.msg_info import current_release, descriptions_release

cr = current_release()
Name = cr[0]
name = cr[1]
Version = cr[2]
Release = cr[3]
Copyright = cr[4]
Website = cr[5]
Author = cr[6]
Mail = cr[7]
Comment = cr[8]

dr = descriptions_release()
Short_Dscrp = dr[0]
Long_Dscrp = dr[1]
Short_Lic = dr[2]
Long_Lic = dr[3]

def info(parent, videomass_icon):
        """
        It's a predefined template to create a dialogue on 
        the program information
        
        """
        info =  wx.adv.AboutDialogInfo()

        info.SetIcon(wx.Icon(videomass_icon, type=wx.BITMAP_TYPE_PNG))
        #info.SetIcon(videomass_icon)
        
        info.SetName("%s" % Name)
        
        info.SetVersion("%s" % Version)
        
        info.SetDescription(_("Videomass provides a graphical interface for\n"
                              "audio and video conversion through FFmpeg\n"
                              "\n"
                              "It is based on Python3 and wxPython4 phoenix")
                            )
        info.SetCopyright("Copyright %s %s" %(Copyright, Author))
        
        info.SetWebSite(Website)
        
        info.SetLicence(Long_Lic)
        
        #info.AddDeveloper("\n%s \n"
                        #"%s\n"
                        #"%s\n\n"
                        #"%s\n" %(Author,Mail,Website,Comment))
        #info.SetDevelopers('')
        info.AddDeveloper("%s %s\n" %(Author,Mail))
        info.AddDeveloper("Thanks to:")
        info.AddDeveloper("- Python (programming language)\n<https://www.python.org/>")
        info.AddDeveloper("- wxPython "
                          "(cross-platform GUI toolkit for the Python language)"
                          "\n<https://wxpython.org/>")
        info.AddDeveloper("- FFmpeg (a complete, cross-platform solution for media)\n"
                           "<http://ffmpeg.org/>")
        info.AddDeveloper("- Material design icons\n"
               "http://google.github.io/material-design-icons/#getting-icons")
        info.AddDeveloper("- Flat Color Icons\nhttp://icons8.com/color-icons\n")
        
        #info.SetDocWriters('')
        info.AddDocWriter("Gianluca Pernigotto (online documentation)\n")
                        
        #info.AddArtist('Gianluca Pernigotto powered by wx.Python')
        
        #info.SetTranslators('')
        info.AddTranslator("Gianluca Pernigotto <jeanlucperni@gmail.com> "
                           "English to Italian translations.\n")
        #info.SetArtists('')
        #info.AddArtist('- Gianluca Pernigotto (Videomass app icons only)\nhttp://jeanslack.github.io/Videomass/')
        #info.AddArtist('- Material design icons from Google\nhttp://google.github.io/material-design-icons/#getting-icons')
        #info.AddArtist('- Flat Color Icons\nhttp://icons8.com/color-icons')
                    
        wx.adv.AboutBox(info)
        #event.Skip()
